Package: rpcss
Title: Constitution of Core Collections by Principal Component Scoring
        Strategy
Version: 0.1.1
Authors@R: c(
    person(given = "J.", family = "Aravind",
           email = "j.aravind@icar.org.in",
           role = c("aut", "cre"),
		       comment = c(ORCID = "0000-0002-4791-442X")),
    person(given = c("Anju", "Mahendru"), family = "Singh",
           email = "anju.singh@icar.org.in",
		       role = "aut",
		       comment = c(ORCID = "0000-0001-6958-1630")),
    person("ICAR-NBGPR",
           role = c("cph"),
		       comment = c(ROR = "https://ror.org/00scbd467",
		                   url = "https://nbpgr.org.in/")))
Description: Generate a Core Collection with Principal Component Scoring 
    Strategy (PCSS) using qualitative and/or quantitative trait data according 
    to Hamon and Noirot (1990) <https://www.documentation.ird.fr/hor/fdi:36506>, 
    Noirot et al. (1996) <doi:10.2307/2527837> and Noirot et al. (2003) 
    <https://www.documentation.ird.fr/hor/fdi:010031886>.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 3.5.0)
RoxygenNote: 7.3.3
BuildManual: TRUE
Imports: dplyr, FactoMineR, ggplot2, ggrepel, gslnls, mathjaxr,
        methods, Rdpack, stats, tidyr, utils
Suggests: EvaluateCore, factoextra, knitr, rmarkdown, pander
RdMacros: mathjaxr, Rdpack
Copyright: 2024-2026, ICAR-NBPGR
URL: https://github.com/aravind-j/rpcss
        https://cran.r-project.org/package=rpcss
        https://aravind-j.github.io/rpcss/
        https://doi.org/10.5281/zenodo.14889174
BugReports: https://github.com/aravind-j/rpcss/issues
NeedsCompilation: no
Packaged: 2026-04-17 09:08:45 UTC; Aravind-DGC
Author: J. Aravind [aut, cre] (ORCID: <https://orcid.org/0000-0002-4791-442X>),
  Anju Mahendru Singh [aut] (ORCID:
    <https://orcid.org/0000-0001-6958-1630>),
  ICAR-NBGPR [cph] (ROR: <https://ror.org/00scbd467>, url:
    https://nbpgr.org.in/)
Maintainer: J. Aravind <j.aravind@icar.org.in>
Repository: CRAN
Date/Publication: 2026-04-29 11:50:02 UTC
Built: R 4.6.0; ; 2026-05-01 22:48:53 UTC; unix

Package: rpcss
Title: Constitution of Core Collections by Principal Component Scoring
        Strategy
Version: 0.1.1
Authors@R: c(
    person(given = "J.", family = "Aravind",
           email = "j.aravind@icar.org.in",
           role = c("aut", "cre"),
		       comment = c(ORCID = "0000-0002-4791-442X")),
    person(given = c("Anju", "Mahendru"), family = "Singh",
           email = "anju.singh@icar.org.in",
		       role = "aut",
		       comment = c(ORCID = "0000-0001-6958-1630")),
    person("ICAR-NBGPR",
           role = c("cph"),
		       comment = c(ROR = "https://ror.org/00scbd467",
		                   url = "https://nbpgr.org.in/")))
Description: Generate a Core Collection with Principal Component Scoring 
    Strategy (PCSS) using qualitative and/or quantitative trait data according 
    to Hamon and Noirot (1990) <https://www.documentation.ird.fr/hor/fdi:36506>, 
    Noirot et al. (1996) <doi:10.2307/2527837> and Noirot et al. (2003) 
    <https://www.documentation.ird.fr/hor/fdi:010031886>.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 3.5.0)
RoxygenNote: 7.3.3
BuildManual: TRUE
Imports: dplyr, FactoMineR, ggplot2, ggrepel, gslnls, mathjaxr,
        methods, Rdpack, stats, tidyr, utils
Suggests: EvaluateCore, factoextra, knitr, rmarkdown, pander
RdMacros: mathjaxr, Rdpack
Copyright: 2024-2026, ICAR-NBPGR
URL: https://github.com/aravind-j/rpcss
        https://cran.r-project.org/package=rpcss
        https://aravind-j.github.io/rpcss/
        https://doi.org/10.5281/zenodo.14889174
BugReports: https://github.com/aravind-j/rpcss/issues
NeedsCompilation: no
Packaged: 2026-04-17 09:08:45 UTC; Aravind-DGC
Author: J. Aravind [aut, cre] (ORCID: <https://orcid.org/0000-0002-4791-442X>),
  Anju Mahendru Singh [aut] (ORCID:
    <https://orcid.org/0000-0001-6958-1630>),
  ICAR-NBGPR [cph] (ROR: <https://ror.org/00scbd467>, url:
    https://nbpgr.org.in/)
Maintainer: J. Aravind <j.aravind@icar.org.in>
Repository: CRAN
Date/Publication: 2026-04-29 11:50:02 UTC
Built: R 4.4.3; ; 2026-05-01 15:31:09 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(contrib,default)
S3method(contrib,pcss.core)
S3method(coreplot,default)
S3method(coreplot,pcss.core)
S3method(print,pcss.core)
S3method(rpcss::biplot,pcss.core)
S3method(rpcss::screeplot,pcss.core)
S3method(rpcss::subset,pcss.core)
export(contrib)
export(coreplot)
export(pcss.core)
import(ggplot2)
import(gslnls)
import(mathjaxr)
importFrom(FactoMineR,CA)
importFrom(FactoMineR,FAMD)
importFrom(FactoMineR,PCA)
importFrom(Rdpack,reprompt)
importFrom(dplyr,bind_rows)
importFrom(ggrepel,geom_label_repel)
importFrom(methods,is)
importFrom(stats,biplot)
importFrom(stats,coef)
importFrom(stats,screeplot)
importFrom(tidyr,pivot_longer)
importFrom(utils,combn)
importFrom(utils,data)

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Fetch the names of individuals/genotypes in the core set generated from
#' \code{pcss.core} Output
#'
#' \code{subset.pcss.core} returns names of individuals/genotypes in the core
#' collection from \code{pcss.core} Output.
#'
#' Use \code{"size"} to return names of individuals/genotypes in the core
#' collection according to the threshold \code{size} criterion or use
#' \code{"variance"} to return names according to the variability threshold
#' criterion or use  \code{"logistic"} to return names according to inflection
#' point of rate of progress of cumulative variability retained identified by
#' logistic regression.
#'
#' @param x An object of class \code{pcss.core}.
#' @param criterion The core collection generation criterion. Either
#'   \code{"size"}, \code{"variance"}, or \code{"logistic"}. See
#'   \strong{Details}.
#' @param ... Unused.
#'
#' @return The names of individuals/genotypes in the core collection as a
#'   character vector.
#'
#' @seealso \code{\link[rpcss]{pcss.core}}
#'
#' @importFrom methods is
#' @exportS3Method rpcss::subset
#'
#' @examples
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' if (requireNamespace('EvaluateCore', quietly = TRUE)) {
#'
#'   suppressPackageStartupMessages(library(EvaluateCore))
#'
#'   # Get data from EvaluateCore
#'
#'   data("cassava_EC", package = "EvaluateCore")
#'   data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
#'   quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
#'              "AVPW", "ARSR", "SRDM")
#'   qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'             "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'             "PSTR")
#'   rownames(data) <- NULL
#'
#'   # Convert qualitative data columns to factor
#'   data[, qual] <- lapply(data[, qual], as.factor)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # With quantitative data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out1 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = NULL, eigen.threshold = NULL, size = 0.2,
#'                     var.threshold = 0.75)
#'
#'   # Core sets
#'   out1$cores.info
#'
#'   # Fetch genotype names of core set by size criterion
#'   subset(x = out1, criterion = "size")
#'
#'   # Fetch genotype names of core set by variance criterion
#'   subset(x = out1, criterion = "variance")
#'
#'   # Fetch genotype names of core set by logistic regression criterion
#'   subset(x = out1, criterion = "logistic")
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
#'                     qualitative = qual, eigen.threshold = NULL,
#'                     size = 0.2, var.threshold = 0.75)
#'
#'   # Core sets
#'   out2$cores.info
#'
#'   # Fetch genotype names of core set by size criterion
#'   subset(x = out2, criterion = "size")
#'
#'   # Fetch genotype names of core set by variance criterion
#'   subset(x = out2, criterion = "variance")
#'
#'   # Fetch genotype names of core set by logistic regression criterion
#'   subset(x = out2, criterion = "logistic")
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (quantitative and qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out3 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = qual, eigen.threshold = NULL)
#'
#'   # Core sets
#'   out3$cores.info
#'
#'   # Fetch genotype names of core set by size criterion
#'   subset(x = out3, criterion = "size")
#'
#'   # Fetch genotype names of core set by variance criterion
#'   subset(x = out3, criterion = "variance")
#'
#'   # Fetch genotype names of core set by logistic regression criterion
#'   subset(x = out3, criterion = "logistic")
#'
#' } else {
#'   message('Package "EvaluateCore" is required to run these examples.')
#' }
#'
subset.pcss.core <- function(x,
                             criterion = c("size", "variance", "logistic"),
                             ...) {

  # Checks ----

  # Check class of "x"
  if (!is(x, "pcss.core")) {
    stop('"x" is not of class "pcss.core".')
  }

  criterion <- match.arg(criterion)

  gssdf <- x$variability.ret

  # By size specified ----

  if (criterion == "size") {
    size.sel <-
      x$cores.info[x$cores.info$Method == "By size specified", ]$Size

    subset <- gssdf[1:size.sel, ]$Id

    if (!is.null(x$always.selected)) {
      always.selected <- x$always.selected

      incl_ind <-
        !(always.selected %in%
            x$variability.ret[(size.sel + 1):nrow(x$variability.ret), "Id"])

      if (any(!incl_ind)) {
        # selected excluding always.sel
        sel.gssdf <- gssdf[1:size.sel, ]
        sel.gssdf <- sel.gssdf[!(sel.gssdf$Id %in% always.selected), ]

        # always.sel
        alsel.gssdf <- gssdf[gssdf$Id %in% always.selected, ]

        # remove from sel to accomodate always.sel
        n.rem <- (nrow(sel.gssdf) + nrow(alsel.gssdf)) - size.sel

        sel.gssdf <- sel.gssdf[1:(nrow(sel.gssdf) - n.rem), ]

        # New sel
        sel.gssdf <- rbind(sel.gssdf, alsel.gssdf)
        sel.gssdf <- sel.gssdf[order(sel.gssdf$Rank), ]

        subset <- sel.gssdf$Id
      }
    }
  }

  # By threshold variance ----

  if (criterion == "variance") {
    var.sel <-
      x$cores.info[x$cores.info$Method == "By threshold variance", ]$Size

    subset <- gssdf[1:var.sel, ]$Id

    if (!is.null(x$always.selected)) {
      always.selected <- x$always.selected

      incl_ind <-
        !(always.selected %in%
            x$variability.ret[(var.sel + 1):nrow(x$variability.ret), "Id"])

      if (any(!incl_ind)) {

        var.threshold <-
          x$cores.info[x$cores.info$Method == "By threshold variance", ]$VarRet

        var.threshold2 <- var.threshold
        var.sel2 <- var.sel

        # always.sel
        alsel.gssdf <- gssdf[gssdf$Id %in% always.selected, ]
        alsel.gssdf$VarRet2 <- (cumsum(alsel.gssdf$CRi) / max(cumCRi)) * 100

        alsel.var <- alsel.gssdf[length(always.selected), ]$VarRet2

        # threshold without contribution by always.sel
        var.threshold <- var.threshold - alsel.var

        # new VarRet wihout always.sel
        gssdf.wo.alsel <- gssdf[!(gssdf$Id %in% always.selected), ]
        gssdf.wo.alsel$VarRet2 <- (cumsum(gssdf.wo.alsel$CRi) /
                                     max(cumCRi)) * 100

        # updated var.sel
        var.sel <- max(which(gssdf.wo.alsel$VarRet2 <= var.threshold))

        var.threshold <-
          gssdf[which(gssdf$Id == gssdf.wo.alsel$Id[var.sel]), "VarRet"]
        # var.sel <- gssdf[which(gssdf$Id == gssdf.wo.alsel$Id[var.sel]),
        #                 "Rank"]

        var.sel2 <-
          var.sel +
          sum(gssdf[(var.sel + 1):nrow(gssdf), ]$Id %in% always.selected)

        rest.Ids <- gssdf[(var.sel + 1):nrow(gssdf), ]$Id

        subset <-
          c(gssdf[1:which(gssdf$Id == gssdf.wo.alsel$Id[var.sel]), "Id"],
            rest.Ids[rest.Ids %in% always.selected])

      }
    }
  }

  # With logistic regression ----

  if (criterion == "logistic") {
    reg.sel <-
      x$cores.info[x$cores.info$Method == "By logistic regression", ]$Size

    subset <- gssdf[1:reg.sel, ]$Id

    if (!is.null(x$always.selected)) {
      always.selected <- x$always.selected

      incl_ind <-
        !(always.selected %in%
            x$variability.ret[(reg.sel + 1):nrow(x$variability.ret), "Id"])

      if (any(!incl_ind)) {

        # selected excluding always.sel
        sel.gssdf <- gssdf[1:reg.sel, ]
        sel.gssdf <- sel.gssdf[!(sel.gssdf$Id %in% always.selected), ]

        # always.sel
        alsel.gssdf <- gssdf[gssdf$Id %in% always.selected, ]

        # remove from sel to accomodate always.sel
        n.rem <- (nrow(sel.gssdf) + nrow(alsel.gssdf)) - reg.sel

        sel.gssdf <- sel.gssdf[1:(nrow(sel.gssdf) - n.rem), ]

        # New sel
        sel.gssdf <- rbind(sel.gssdf, alsel.gssdf)
        sel.gssdf <- sel.gssdf[order(sel.gssdf$Rank), ]

        subset <- sel.gssdf$Id

      }

    }
  }

  return(subset)

}

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Plot Contribution or Loadings of Traits for each Dimension/Factor from
#' \code{pcss.core} Output
#'
#' \code{contrib.pcss.core} generates bar plots of contributions or loadings
#' ("right singular vectors") of traits for each dimension/factor from the
#' output of \code{pcss.core}.
#'
#' @param x An object of class \code{pcss.core}.
#' @param ndim The number of dimensions for which contribution or loadings of
#'   traits are to be plotted.
#' @param plot.loadings If \code{TRUE}, the loadings or "right singular vectors"
#'   are plotted instead of contributions. Default is \code{FALSE}.
#' @param use.sign If \code{TRUE}, contributions of variables are given the sign
#'   of their corresponding coordinates. Default is \code{TRUE}.
#' @param sort.value If \code{TRUE}, the bars are sorted according to their
#'   value.
#' @param ... Unused.
#'
#' @return The contributions/loadings bar plot as a \code{ggplot} object.
#'
#' @seealso \code{\link[rpcss]{pcss.core}}
#'
#' @import ggplot2
#' @importFrom tidyr pivot_longer
#' @export
#'
#' @examples
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' if (requireNamespace('EvaluateCore', quietly = TRUE) &
#'     requireNamespace("factoextra", quietly = TRUE)) {
#'
#'   # Get data from EvaluateCore
#'
#'   data("cassava_EC", package = "EvaluateCore")
#'   data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
#'   quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
#'              "ARSR", "SRDM")
#'   qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'             "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'             "PSTR")
#'   rownames(data) <- NULL
#'
#'   # Convert qualitative data columns to factor
#'   data[, qual] <- lapply(data[, qual], as.factor)
#'
#'
#'   library(FactoMineR)
#'   suppressPackageStartupMessages(library(factoextra))
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # With quantitative data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out1 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = NULL, eigen.threshold = NULL, size = 0.2,
#'                     var.threshold = 0.75)
#'
#'   # Plot contributions of genotypes - with sign - sorted
#'   contrib(x = out1, ndim = 5)
#'
#'   # Plot contributions of genotypes - without sign - sorted
#'   contrib(x = out1, ndim = 5, use.sign = FALSE)
#'
#'   # Plot loadings/coordinates of genotypes - with sign - sorted
#'   contrib(x = out1, ndim = 5, plot.loadings = TRUE)
#'
#'   # Plot contributions of genotypes - with sign - unsorted
#'   contrib(x = out1, ndim = 5, sort.value = FALSE)
#'
#'   # Plot biplot with factoextra
#'   fviz_contrib(out1$raw.out, choice = "var", axes = 1)
#'   fviz_contrib(out1$raw.out, choice = "var", axes = 2)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
#'                     qualitative = qual, eigen.threshold = NULL,
#'                     size = 0.2, var.threshold = 0.75)
#'
#'   # Plot contributions of genotypes - with sign - sorted
#'   contrib(x = out2, ndim = 5)
#'
#'   # Plot contributions of genotypes - without sign - sorted
#'   contrib(x = out2, ndim = 5, use.sign = FALSE)
#'
#'   # Plot loadings/coordinates of genotypes - with sign - sorted
#'   contrib(x = out2, ndim = 5, plot.loadings = TRUE)
#'
#'   # Plot contributions of genotypes - with sign - unsorted
#'   contrib(x = out2, ndim = 5, sort.value = FALSE)
#'
#'   # Plot biplot with factoextra
#'   fviz_contrib(out2$raw.out, choice = "var", axes = 1)
#'   fviz_contrib(out2$raw.out, choice = "var", axes = 2)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (quantitative and qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out3 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = qual, eigen.threshold = NULL)
#'
#'   # Plot contributions of genotypes - sorted
#'   contrib(x = out3, ndim = 5)
#'
#'   # Plot contributions of genotypes - without sign - sorted
#'   contrib(x = out3, ndim = 5, use.sign = FALSE)
#'
#'   # Plot loadings/coordinates of genotypes - sorted
#'   contrib(x = out3, ndim = 5, plot.loadings = TRUE)
#'
#'   # Plot contributions of genotypes - with sign - unsorted
#'   contrib(x = out3, ndim = 5, sort.value = FALSE)
#'
#'   # Plot biplot with factoextra
#'   # Has a bug (https://github.com/kassambara/factoextra/issues/184)
#'   # fviz_contrib(out3$raw.out, choice = "quanti.var", axes = 1)
#'   # fviz_contrib(out3$raw.out, choice = "quali.var", axes = 1)
#'   # fviz_contrib(out3$raw.out, choice = "quanti.var", axes = 2)
#'   # fviz_contrib(out3$raw.out, choice = "quali.var", axes = 2)
#'
#' } else {
#'   message('Packages "EvaluateCore" and "factoextra" are required to run these examples.')
#' }
#'
contrib <- function(x, ndim = NULL,
                    plot.loadings = FALSE,
                    use.sign = TRUE,
                    sort.value = TRUE, ...) {
  UseMethod("contrib")
}

#' @name contrib
#' @method contrib default
#' @export
contrib.default <- function(x, ndim = NULL,
                            plot.loadings = FALSE,
                            use.sign = TRUE,
                            sort.value = TRUE, ...) {

  contrib.pcss.core(x, ndim = NULL,
                    plot.loadings = FALSE,
                    use.sign = TRUE,
                    sort.value = TRUE)
}

#' @name contrib
#' @export
contrib.pcss.core <- function(x, ndim = NULL,
                              plot.loadings = FALSE,
                              use.sign = TRUE,
                              sort.value = TRUE, ...) {
  # Checks ----

  # Check class of "x"
  if (!is(x, "pcss.core")) {
    stop('"x" is not of class "pcss.core".')
  }

  if (!is.null(ndim)) {
    # check if 'ndim' argument is integer vector of unit length
    if (!(as.integer(ndim) == ndim && length(ndim) == 1)) {
      stop('"ndim" should be a integer vector of unit length.')
    }

    # check if at least one dimension are plotted in ndim
    if (ndim < 1L) {
      stop('At least 1 dimension is to be specified in "ndim".')
    }

    # check if ndim is not greater than total dimensions
    total.ndim <- ncol(x$raw.out$svd$V)

    if (ndim > total.ndim) {
      warning('"ndim" is greater than the total number of dimensions.\n',
              paste('Using the total number of dimensions (',
                    total.ndim, ') as "ndim".', sep = ""))
      ndim <- total.ndim
    }

  }

  method <- attr(x, "method")

  if (plot.loadings) {

    pdata <- x$raw.out$svd$V

    if (method != "FAMD") {

      rownames(pdata) <- rownames(x$raw.out$var$coord)
      colnames(pdata) <- colnames(x$raw.out$var$coord)

    } else {

      quali.levels <- attr(x, "quali.levels")
      quant <- attr(x, "quant")

      rownames(pdata) <- c(quant,
                           unlist(
                             lapply(seq_along(quali.levels), function(i) {
                               paste(names(quali.levels)[i], quali.levels[[i]],
                                     sep = "_")
                             })
                           ))
      colnames(pdata) <- gsub("Dim.", "Dim ", colnames(pdata))
    }


  } else {


    if (method != "FAMD") {

      pdata <- x$raw.out$var$contrib

      if (use.sign) {
        pdata <- pdata * sign(x$raw.out$var$coord)
      }

      rownames(pdata) <- rownames(x$raw.out$var$coord)
      colnames(pdata) <- colnames(x$raw.out$var$coord)

    } else {

      pdata <- rbind(x$raw.out$quanti.var$contrib,
                     x$raw.out$quali.var$contrib)

      if (use.sign) {
        pdata <- pdata *  rbind(sign(x$raw.out$quanti.var$coord),
                                sign(x$raw.out$quali.var$coord))
      }

      quali.levels <- attr(x, "quali.levels")
      quant <- attr(x, "quant")

      rownames(pdata) <-
        c(quant,
          unlist(
            lapply(seq_along(quali.levels), function(i) {
              paste(names(quali.levels)[i], quali.levels[[i]], sep = "_")
            })
          ))
      colnames(pdata) <- gsub("Dim.", "Dim ", colnames(pdata))

    }

  }

  pdata <- data.frame(pdata, check.rows = FALSE)

  eigdf <- x$eigen

  if (!is.null(ndim)) {
    if (nrow(eigdf) > ndim) {
      pdata <- pdata[, 1:ndim]
    }
  }

  pdata$Trait <- rownames(pdata)

  pdata_long <- tidyr::pivot_longer(data = pdata, cols = !Trait,
                                    names_to = "Dimension",
                                    values_to = "Value")
  pdata_long$Dimension <- as.factor(as.integer(gsub("\\D", "",
                                                    pdata_long$Dimension)))
  levels(pdata_long$Dimension) <- paste("Dim", levels(pdata_long$Dimension))

  pdata_long$Group <- ifelse(pdata_long$Value >= 0, TRUE, FALSE)

  pdata_long <- pdata_long[rev(order(pdata_long$Value)), ]
  pdata_long$Trait_Dim <- paste(pdata_long$Trait,
                                pdata_long$Dimension, sep = "#")
  pdata_long$Trait_Dim <- factor(pdata_long$Trait_Dim,
                                 levels = rev(paste(pdata_long$Trait,
                                                    pdata_long$Dimension,
                                                    sep = "#")))

  if (sort.value) {

    contribg <-
      ggplot(pdata_long, aes(y = Trait_Dim, x = Value, fill = Group)) +
      geom_bar(stat = "identity", show.legend = FALSE,
               colour = "transparent") +
      facet_wrap(vars(Dimension), scales = "free_y") +
      scale_y_discrete(labels = function(x) gsub("#.+$", "", x)) +
      ylab("Trait") +
      theme_bw()

  } else {

    contribg <-
      ggplot(pdata_long, aes(y = Trait, x = Value, fill = Group)) +
      geom_bar(stat = "identity", show.legend = FALSE,
               colour = "transparent") +
      facet_wrap(vars(Dimension), scales = "free_y") +
      theme_bw()
  }

  if (any(pdata_long$Value < 0)) {

    contribg <- contribg +
      scale_fill_manual(values = c("firebrick2", "steelblue3")) +
      geom_vline(xintercept = 0)


  } else {

    contribg <- contribg +
      scale_fill_manual(values = c("gray10", "gray10")) +
      geom_vline(xintercept = 0)

  }

  return(contribg)

}

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Prints summary of \code{pcss.core} object
#'
#' \code{pcss.core} prints to console the summary of an object of class
#' \code{pcss.core} including the dimensionality reduction method used, the
#' basic details including parameters and the information on the core sets that
#' can be constituted.
#'
#' @param x An object of class \code{pcss.core}.
#' @param ... Unused.
#' @seealso \code{\link[rpcss]{pcss.core}}
#'
#' @return The argument x, invisibly as for all \code{\link[base]{print}}
#'   methods.
#'
#' @export
#'
print.pcss.core <- function(x, ...) {

  cat("\nMethod\n")
  cat("========================\n")

  print(attr(x, "method"))

  cat("\nDetails\n")
  cat("========================\n")

  print(x$details)

  cat("\nCore sets\n")
  cat("=========================\n")

  print(x$cores.info)

}

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Plot Eigen values as a Scree Plot from \code{pcss.core} Output
#'
#' \code{screeplot.pcss.core} generates a scree plot of eigen values from the
#' output of \code{pcss.core}.
#'
#' @param x An object of class \code{pcss.core}.
#' @param ndim The number of eigen values to be plotted in the scree plot.
#' @param show.values If \code{TRUE}, the eigen values are shown in the plot as
#'   annotation labels. Default is \code{TRUE}.
#' @param ... Unused.
#'
#' @return The scree plot as a \code{ggplot} object.
#'
#' @seealso \code{\link[rpcss]{pcss.core}},
#'   \code{\link[factoextra]{fviz_screeplot}}
#'
#' @import ggplot2
#' @importFrom stats screeplot
#' @exportS3Method rpcss::screeplot
#'
#' @examples
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' if (requireNamespace('EvaluateCore', quietly = TRUE) &
#'     requireNamespace("factoextra", quietly = TRUE)) {
#'
#'   # Get data from EvaluateCore
#'
#'   data("cassava_EC", package = "EvaluateCore")
#'   data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
#'   quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
#'              "ARSR", "SRDM")
#'   qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'             "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'             "PSTR")
#'   rownames(data) <- NULL
#'
#'   # Convert qualitative data columns to factor
#'   data[, qual] <- lapply(data[, qual], as.factor)
#'
#'
#'   library(FactoMineR)
#'   suppressPackageStartupMessages(library(factoextra))
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # With quantitative data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out1 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = NULL, eigen.threshold = NULL, size = 0.2,
#'                     var.threshold = 0.75)
#'
#'   # Plot scree plot
#'   screeplot(x = out1)
#'
#'   # Plot biplot with factoextra
#'   fviz_screeplot(out1$raw.out)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
#'                     qualitative = qual, eigen.threshold = NULL,
#'                     size = 0.2, var.threshold = 0.75)
#'
#'   # Plot scree plot
#'   screeplot(x = out2)
#'
#'   # Plot biplot with factoextra
#'   fviz_screeplot(out2$raw.out)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (quantitative and qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out3 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = qual, eigen.threshold = NULL)
#'
#'   # Plot scree plot
#'   screeplot(x = out3)
#'
#'   # Plot biplot with factoextra
#'   fviz_screeplot(out3$raw.out)
#'
#' } else {
#'   message('Packages "EvaluateCore" and "factoextra" are required to run these examples.')
#' }
#'
screeplot.pcss.core <- function(x, ndim = NULL, show.values = TRUE, ...) {

  # Checks ----

  # Check class of "x"
  if (!is(x, "pcss.core")) {
    stop('"x" is not of class "pcss.core".')
  }

  if (!is.null(ndim)) {
    # check if 'ndim' argument is integer vector of unit length
    if (!(as.integer(ndim) == ndim && length(ndim) == 1)) {
      stop('"ndim" should be a integer vector of unit length.')
    }

    # check if at least one dimension are plotted in ndim
    if (ndim < 1L) {
      stop('At least 1 dimension is to be specified in "ndim".')
    }

    # check if ndim is not greater than total dimensions
    total.ndim <- ncol(x$raw.out$svd$V)

    if (ndim > total.ndim) {
      warning('"ndim" is greater than the total number of dimensions.\n',
              paste('Using the total number of dimensions (',
                    total.ndim, ') as "ndim".', sep = ""))
      ndim <- total.ndim
    }

  }

  eigen.threshold <- x$eigen.threshold
  eig <- x$eigen$`Eigen value`

  eigen.threshold_label <- eigen.threshold
  if (nchar(eigen.threshold > 5)) {
    eigen.threshold_label <- round(eigen.threshold, 3)
  }

  K <- x$details[x$details$Detail == "Number of eigen values selected", "Value"]
  K <- as.numeric(K)

  # Plot eigen values ----
  eigdf <- data.frame(sl = seq_along(eig), eig = eig)
  eigdf$gp <- ifelse(eigdf$sl <= K, 1, 0)
  eigdf$gp <- as.factor(eigdf$gp)

  if (!is.null(ndim)) {
    if (nrow(eigdf) > ndim) {
      eigdf <- eigdf[1:ndim, ]
    }
  }

  eigg <- ggplot(eigdf) +
    geom_segment(aes(x = sl, y = 0, xend = sl, yend = eig),
                 linewidth = 2, colour = "gray20") +
    geom_line(aes(x = sl, y = eig)) +
    geom_point(aes(x = sl, y = eig, colour = gp), pch = 18, size = 3,
               show.legend = FALSE) +
    scale_colour_manual(values = c("gray", "red")) +
    geom_hline(yintercept = eigen.threshold, linetype = 2) +
    geom_label(x = length(eig), y = eigen.threshold, colour = "red",
               label = eigen.threshold_label) +
    xlab("Factors") +
    ylab("Eigen value") +
    theme_bw()

  if (show.values) {
    eigg <- eigg +
      geom_label(aes(x = sl, y = eig, label = round(eig, 3)),
                 vjust = 1, hjust = -0.2) +
      scale_x_continuous(breaks = eigdf$sl,
                         limits = c(1, length(eig) + 0.5)) +
      scale_y_continuous(expand = expansion(mult = c(0.1, 0.05)))

  }

  return(eigg)
}

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Plot the cumulative variability retained by individuals/genotypes from
#' \code{pcss.core} Output
#'
#' \code{coreplot.pcss.core} generates plots of cumulative variability retained
#' by individuals/genotypes from \code{pcss.core} Output. The size of core
#' collection and the corresponding cumulative variance retained are highlighted
#' according to the criterion used.
#'
#' Use \code{"size"} to highlight core collection according to the threshold
#' \code{size} criterion or use \code{"variance"} to highlight core collection
#' according to the variability threshold criterion or use  \code{"logistic"} to
#' highlight core collection generated according to inflection point of rate of
#' progress of cumulative variability retained identified by logistic
#' regression.
#'
#' @param x An object of class \code{pcss.core}.
#' @param criterion The core collection generation criterion. Either
#'   \code{"size"}, \code{"variance"}, or \code{"logistic"}. See
#'   \strong{Details}.
#' @param ... Unused.
#'
#' @return A plot of cumulative variability retained by individuals/genotypes as
#'   a \code{ggplot} object. In case of \code{criterion = "logistic"}, a list
#'   with plots of cumulative variability retained by individuals/genotypes and
#'   rate of progress of cumulative contribution to variability. The size and
#'   variability retained by core collection are highlighted in each plot.
#'
#' @seealso \code{\link[rpcss]{pcss.core}}
#'
#' @import ggplot2
#' @export
#'
#' @examples
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' if (requireNamespace('EvaluateCore', quietly = TRUE)) {
#'
#'   suppressPackageStartupMessages(library(EvaluateCore))
#'
#'   # Get data from EvaluateCore
#'
#'   data("cassava_EC", package = "EvaluateCore")
#'   data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
#'   quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
#'              "AVPW", "ARSR", "SRDM")
#'   qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'             "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'             "PSTR")
#'   rownames(data) <- NULL
#'
#'   # Convert qualitative data columns to factor
#'   data[, qual] <- lapply(data[, qual], as.factor)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # With quantitative data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out1 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = NULL, eigen.threshold = NULL, size = 0.2,
#'                     var.threshold = 0.75)
#'
#'   # For core set constituted by size criterion
#'   coreplot(x = out1, criterion = "size")
#'
#'   # For core set constituted by variance criterion
#'   coreplot(x = out1, criterion = "variance")
#'
#'   # For core set constituted by logistic regression criterion
#'   coreplot(x = out1, criterion = "logistic")
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
#'                     qualitative = qual, eigen.threshold = NULL,
#'                     size = 0.2, var.threshold = 0.75)
#'
#'   # For core set constituted by size criterion
#'   coreplot(x = out2, criterion = "size")
#'
#'   # For core set constituted by variance criterion
#'   coreplot(x = out2, criterion = "variance")
#'
#'   # For core set constituted by logistic regression criterion
#'   coreplot(x = out2, criterion = "logistic")
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (quantitative and qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out3 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = qual, eigen.threshold = NULL)
#'
#'   # For core set constituted by size criterion
#'   coreplot(x = out3, criterion = "size")
#'
#'   # For core set constituted by variance criterion
#'   coreplot(x = out3, criterion = "variance")
#'
#'   # For core set constituted by logistic regression criterion
#'   coreplot(x = out3, criterion = "logistic")
#'
#' } else {
#'   message('Package "EvaluateCore" is required to run these examples.')
#' }
#'
coreplot <- function(x,
                     criterion = c("size", "variance", "logistic"),
                     ...) {
  UseMethod("coreplot")
}

#' @name coreplot
#' @method coreplot default
#' @export
coreplot.default <- function(x,
                             criterion = c("size", "variance", "logistic"),
                             ...) {

  coreplot.pcss.core(x,
                     criterion = c("size", "variance", "logistic"),
                     ...)
}

#' @name coreplot
#' @export
coreplot.pcss.core <- function(x,
                               criterion = c("size", "variance", "logistic"),
                               ...) {

  # Checks ----

  # Check class of "x"
  if (!is(x, "pcss.core")) {
    stop('"x" is not of class "pcss.core".')
  }

  criterion <- match.arg(criterion)

  gssdf <- x$variability.ret
  cumCRi <- cumsum(gssdf$CRi)

  N <- nrow(gssdf)

  propname <- "Proportion of selected individuals"

  # By size specified ----

  if (criterion == "size") {

    size.sel <-
      x$cores.info[x$cores.info$Method == "By size specified", ]$Size
    size.var <-
      x$cores.info[x$cores.info$Method == "By size specified", ]$VarRet

    if (!is.null(x$always.selected)) {
      always.selected <- x$always.selected

      incl_ind <-
        !(always.selected %in%
            x$variability.ret[(size.sel + 1):nrow(x$variability.ret), "Id"])

      if (any(!incl_ind)) {

        # selected excluding always.sel
        sel.gssdf <- gssdf[1:size.sel, ]
        sel.gssdf <- sel.gssdf[!(sel.gssdf$Id %in% always.selected), ]

        # always.sel
        alsel.gssdf <- gssdf[gssdf$Id %in% always.selected, ]

        # remove from sel to accomodate always.sel
        n.rem <- (nrow(sel.gssdf) + nrow(alsel.gssdf)) - size.sel

        sel.gssdf <- sel.gssdf[1:(nrow(sel.gssdf) - n.rem), ]

        # New sel
        sel.gssdf <- rbind(sel.gssdf, alsel.gssdf)
        sel.gssdf <- sel.gssdf[order(sel.gssdf$Rank), ]

        # New sel including always.sel in orginal size.sel
        acc.sel <- sel.gssdf$Id
        acc.sel <- acc.sel[!(acc.sel %in% always.selected)]
        acc.sel <- c(acc.sel, always.selected[incl_ind])

        size.sel <- length(acc.sel)

        sel.gssdf_a <- gssdf[gssdf$Id %in% acc.sel, ]
        sel.gssdf_a <- sel.gssdf_a[order(sel.gssdf_a$Rank), ]
        sel.gssdf_a$VarRet2 <- (cumsum(sel.gssdf_a$CRi) / max(cumCRi)) * 100

        size.var <- sel.gssdf_a[size.sel, ]$VarRet2

        # # always.sel not in orginal size.sel
        # size.sel2 <- length(always.selected[!incl_ind])
        #
        # sel.gssdf_b <- gssdf[gssdf$Id %in%  always.selected[!incl_ind], ]
        # sel.gssdf_b <- sel.gssdf_b[order(sel.gssdf_b$Rank), ]
        # sel.gssdf_b$VarRet2 <- (cumsum(sel.gssdf_b$CRi) / max(cumCRi)) * 100
        #
        # size.var2 <- sel.gssdf_b[size.sel2, ]$VarRet2

      }

    }

    size.segdf <- data.frame(x = c(-Inf, size.sel),
                             xend = c(size.sel, size.sel),
                             y = c(size.var, -Inf),
                             yend = c(size.var, size.var))
    size.segdf$label <- as.character(c(round(size.segdf[1, "y"], 2),
                                       size.segdf[2, "x"]))

    if (!is.null(x$always.selected)) {
      if (any(!incl_ind)) {

        size.segdf$label <-
          paste(size.segdf$label,
                as.character(c(round(
                  x$cores.info[x$cores.info$Method ==
                                 "By size specified", ]$VarRet, 2),
                  x$cores.info[x$cores.info$Method ==
                                 "By size specified", ]$Size)),
                sep = " / ")
      }
    }

    size.gssg <- ggplot(gssdf, aes(x = Rank, y = VarRet)) +
      geom_point() +
      geom_segment(data = size.segdf, aes(x = x, xend = xend,
                                          y = y, yend = yend),
                   colour = "red") +
      geom_text(data = size.segdf, aes(x = x, y = y, label = label),
                vjust = -0.5, hjust = -0.5, colour = "red") +
      scale_x_continuous(name = "Number of selected individuals",
                         sec.axis = sec_axis(transform = ~. / N,
                                             name = propname)) +
      ylab("Variability retained (%)") +
      theme_bw()

    if (!is.null(x$always.selected)) {
      if (any(!incl_ind)) {

        gssdf2 <- gssdf[gssdf$Id %in% always.selected, ]

        size.gssg <-
          size.gssg +
          geom_point(data = gssdf2, colour = "red")

      }
    }

    return(size.gssg)
  }

  # By threshold variance ----

  if (criterion == "variance") {

    var.sel <-
      x$cores.info[x$cores.info$Method == "By threshold variance", ]$Size
    var.threshold <-
      x$cores.info[x$cores.info$Method == "By threshold variance", ]$VarRet

    if (!is.null(x$always.selected)) {
      always.selected <- x$always.selected

      incl_ind <-
        !(always.selected %in%
            x$variability.ret[(var.sel + 1):nrow(x$variability.ret), "Id"])

      if (any(!incl_ind)) {

        var.threshold2 <- var.threshold
        var.sel2 <- var.sel

        # always.sel
        alsel.gssdf <- gssdf[gssdf$Id %in% always.selected, ]
        alsel.gssdf$VarRet2 <- (cumsum(alsel.gssdf$CRi) / max(cumCRi)) * 100

        alsel.var <- alsel.gssdf[length(always.selected), ]$VarRet2

        # threshold without contribution by always.sel
        var.threshold <- var.threshold - alsel.var

        # new VarRet wihout always.sel
        gssdf.wo.alsel <- gssdf[!(gssdf$Id %in% always.selected), ]
        gssdf.wo.alsel$VarRet2 <- (cumsum(gssdf.wo.alsel$CRi) /
                                     max(cumCRi)) * 100

        # updated var.sel
        var.sel <- max(which(gssdf.wo.alsel$VarRet2 <= var.threshold))

        var.threshold <-
          gssdf[which(gssdf$Id == gssdf.wo.alsel$Id[var.sel]), "VarRet"]
        var.sel <- gssdf[which(gssdf$Id == gssdf.wo.alsel$Id[var.sel]), "Rank"]

        var.sel2 <-
          var.sel +
          sum(gssdf[(var.sel + 1):nrow(gssdf), ]$Id %in% always.selected)

      }

    }

    var.segdf <- data.frame(x = c(-Inf, var.sel),
                            xend = c(var.sel, var.sel),
                            y = c(var.threshold, -Inf),
                            yend = c(var.threshold, var.threshold))
    var.segdf$label <- as.character(c(round(var.segdf[1, "y"], 2),
                                      var.segdf[2, "x"]))

    if (!is.null(x$always.selected)) {
      if (any(!incl_ind)) {

        var.segdf$label <-
          paste(var.segdf$label,
                as.character(c(round(var.threshold2, 2), var.sel2)),
                sep = " / ")
      }
    }

    var.gssg <- ggplot(gssdf, aes(x = Rank, y = VarRet)) +
      geom_point() +
      geom_segment(data = var.segdf, aes(x = x, xend = xend,
                                         y = y, yend = yend),
                   colour = "red") +
      geom_text(data = var.segdf, aes(x = x, y = y, label = label),
                vjust = -0.5, hjust = -0.5, colour = "red") +
      scale_x_continuous(name = "Number of selected individuals",
                         sec.axis = sec_axis(transform = ~. / N,
                                             name = propname)) +
      ylab("Variability retained (%)") +
      theme_bw()

    if (!is.null(x$always.selected)) {
      if (any(!incl_ind)) {

        gssdf2 <- gssdf[gssdf$Id %in% always.selected, ]

        var.gssg <-
          var.gssg +
          geom_point(data = gssdf2, colour = "red")

      }
    }

    return(var.gssg)
  }

  # With logistic regression ----

  if (criterion == "logistic") {

    reg.sel <-
      x$cores.info[x$cores.info$Method == "By logistic regression", ]$Size
    reg.var <-
      x$cores.info[x$cores.info$Method == "By logistic regression", ]$VarRet

    b <- attributes(x)$slope

    if (!is.null(x$always.selected)) {
      always.selected <- x$always.selected

      incl_ind <-
        !(always.selected %in%
            x$variability.ret[(reg.sel + 1):nrow(x$variability.ret), "Id"])

      if (any(!incl_ind)) {

        # selected excluding always.sel
        sel.gssdf <- gssdf[1:reg.sel, ]
        sel.gssdf <- sel.gssdf[!(sel.gssdf$Id %in% always.selected), ]

        # always.sel
        alsel.gssdf <- gssdf[gssdf$Id %in% always.selected, ]

        # remove from sel to accomodate always.sel
        n.rem <- (nrow(sel.gssdf) + nrow(alsel.gssdf)) - reg.sel

        sel.gssdf <- sel.gssdf[1:(nrow(sel.gssdf) - n.rem), ]

        # New sel
        sel.gssdf <- rbind(sel.gssdf, alsel.gssdf)
        sel.gssdf <- sel.gssdf[order(sel.gssdf$Rank), ]

        # New sel including always.sel in orginal reg.sel
        acc.sel <- sel.gssdf$Id
        acc.sel <- acc.sel[!(acc.sel %in% always.selected)]
        acc.sel <- c(acc.sel, always.selected[incl_ind])

        reg.sel <- length(acc.sel)

        sel.gssdf_a <- gssdf[gssdf$Id %in% acc.sel, ]
        sel.gssdf_a <- sel.gssdf_a[order(sel.gssdf_a$Rank), ]
        sel.gssdf_a$VarRet2 <- (cumsum(sel.gssdf_a$CRi) / max(cumCRi)) * 100

        reg.var <- sel.gssdf_a[reg.sel, ]$VarRet2

      }

    }

    reg.segdf <- data.frame(x = c(-Inf, reg.sel),
                            xend = c(reg.sel, reg.sel),
                            y = c(reg.var, -Inf),
                            yend = c(reg.var, reg.var))
    reg.segdf$label <- as.character(c(round(reg.segdf[1, "y"], 2),
                                      reg.segdf[2, "x"]))

    if (!is.null(x$always.selected)) {
      if (any(!incl_ind)) {

        reg.segdf$label <-
          paste(reg.segdf$label,
                as.character(c(round(
                  x$cores.info[x$cores.info$Method ==
                                 "By logistic regression", ]$VarRet, 2),
                  x$cores.info[x$cores.info$Method ==
                                 "By logistic regression", ]$Size)),
                sep = " / ")
      }
    }

    reg.gssg <- ggplot(gssdf, aes(x = Rank, y = VarRet)) +
      geom_point() +
      geom_segment(data = reg.segdf, aes(x = x, xend = xend,
                                         y = y, yend = yend),
                   colour = "red") +
      geom_text(data = reg.segdf, aes(x = x, y = y, label = label),
                vjust = -0.5, hjust = -0.5, colour = "red") +
      scale_x_continuous(name = "Number of selected individuals",
                         sec.axis = sec_axis(transform = ~. / N,
                                             name = propname)) +
      ylab("Variability retained (%)") +
      theme_bw()


    if (!is.null(x$always.selected)) {
      if (any(!incl_ind)) {

        gssdf2 <- gssdf[gssdf$Id %in% always.selected, ]

        reg.gssg <-
          reg.gssg +
          geom_point(data = gssdf2, colour = "red")

      }
    }

    dat <- data.frame(n = gssdf$Rank, y = gssdf$VarRet, Id = gssdf$Id)
    dat$rate <- -b * dat$y * (100 - dat$y)

    reg.segdf2 <- data.frame(x = c(-Inf, reg.sel),
                             xend = c(reg.sel, reg.sel),
                             y = c(max(dat$rate), -Inf),
                             yend = c(max(dat$rate), max(dat$rate)))
    reg.segdf2$label <- as.character(c(round(reg.segdf2[1, "y"], 2),
                                       reg.segdf2[2, "x"]))

    reg.gssrateg <- ggplot(data = dat, aes(x = n, y = rate)) +
      geom_point() +
      geom_segment(data = reg.segdf2, aes(x = x, xend = xend,
                                          y = y, yend = yend),
                   colour = "red") +
      geom_text(data = reg.segdf2, aes(x = x, y = y, label = label),
                vjust = -0.5, hjust = -0.5, colour = "red") +
      scale_x_continuous(name = "Number of selected individuals",
                         sec.axis = sec_axis(transform = ~. / N,
                                             name = propname)) +
      scale_y_continuous(expand = expansion(mult = c(0.05, 0.1))) +
      ylab("Rate of increase in variability retained (%)") +
      theme_bw()

    if (!is.null(x$always.selected)) {
      if (any(!incl_ind)) {

        dat2 <- dat[dat$Id %in% always.selected, ]

        reg.gssrateg <-
          reg.gssrateg +
          geom_point(data = dat2, colour = "red")

      }
    }

    return(list(`Cumulative contribution` = reg.gssg,
                `Rate of cumulative contribution` = reg.gssrateg))
  }

}

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Principal Component Scoring to Generate Core collections
#'
#' Generate a Core Collection with Principal Component Scoring Strategy (PCSS)
#' \insertCite{hamon_proposed_1990,noirot_principal_1996,noirot_method_2003}{rpcss}
#' using qualitative and/or quantitative trait data. \loadmathjax
#'
#' A core collection is constituted from an entire collection of \mjseqn{N}
#' genotypes using quantitative data of \mjseqn{J} traits using Principal
#' Component Scoring Strategy (PCSS)
#' \insertCite{hamon_proposed_1990,noirot_principal_1996,noirot_method_2003}{rpcss}
#' as follows:
#'
#' \enumerate{
#'
#' \item Principal Component Analysis (PCA) is performed on the standardized
#' genotype \mjseqn{\times} trait data. This takes care of multicollinearity
#' between the traits to generate \mjseqn{J} standardized and independent
#' variables or factors or principal component.
#'
#' \item Considering only a subset of factors \mjseqn{K}, the Generalized Sum of
#'  Squares (GSS) of N individuals in K factorial spaces is computed as
#' \mjseqn{N \times K}.
#'
#' \mjseqn{K} can be the number of factors for which the eigen value
#' \mjseqn{\lambda} is greater than a threshold value such as 1 (Kaiser-Guttman
#' criterion) or the average of all the eigen values.
#'
#' \item The contribution of the \mjseqn{i}th genotype to GSS (\mjseqn{P_{i}})
#' or total variability is calculated as below.
#'
#' \mjsdeqn{P_{i} = \sum_{j = 1}^{K} x_{ij}^{2}}
#'
#' Where \mjseqn{x_{ij}} is the component score or coordinate of the
#' \mjseqn{i}th genotype on the \mjseqn{j}th principal component.
#'
#' \item For each genotype, its relative contribution to GSS or total
#' variability is computed as below.
#'
#' \mjsdeqn{CR_{i} = \frac{P_{i}}{N \times K}}
#'
#' \item The genotypes are sorted in descending order of magnitude of their
#' contribution to GSS and then the cumulative contribution of successive
#' genotypes to GSS is computed.
#'
#' \item The core collection can then be selected by three different methods.
#'
#'  \enumerate{
#'
#'  \item Selection of fixed proportion or percentage or number of the top
#'  accessions.
#'
#'  \item Selection of the top accessions that contribute up to a fixed
#'  percentage of the GSS.
#'
#'  \item Fitting a logistic regression model of the following form to the
#'  cumulative contribution of successive genotypes to GSS
#'  \insertCite{balakrishnan_method_2000}{rpcss}.
#'
#'  \mjsdeqn{\frac{y}{A-y} = e^{a + bn}}
#'
#'  The above equation can  be reparameterized as below.
#'
#'  \mjsdeqn{\log_{e} \left ( {\frac{y}{A-y}} \right ) = a + bn}
#'
#'  Where, \mjseqn{a} and \mjseqn{b} are the intercept and regression
#'  coefficient, respectively; \mjseqn{y} is the cumulative contribution of
#'  successive genotypes to GSS; \mjseqn{n} is the rank of the genotype when
#'  sorted according to the contribution to GSS and \mjseqn{A} is the asymptote
#'  of the curve (\mjseqn{A = 100}).
#'
#'  The rate of increase in the successive contribution of genotypes to GSS can
#'  be computed by the following equation to find the point of inflection where
#'  the rate of increase starts declining.
#'
#'  \mjseqn{\frac{\mathrm{d} y}{\mathrm{d} x} = by(A-y)}
#'
#'  The number of accessions included till the peak or infection point are
#'  selected to constitute the core collection.
#'
#'  }
#'
#' }
#'
#' Similarly for qualitative traits, standardized and independent variables or
#' factors can be obtained by Correspondence Analysis (CA) on complete
#' disjunctive table of genotype \mjseqn{\times} trait data or to be specific
#' Multiple Correspondence Analysis (MCA). In \code{rpcss}, this has also been
#' extended for data sets having both quantitative and qualitative traits by
#' implementing Factor Analysis for Mixed Data (FAMD) for obtaining standardized
#' and independent variables or factors.
#'
#' In \code{rpcss}, PCA, MCA and FAMD are implemented via the
#' \code{\link[FactoMineR]{FactoMineR}} package.
#' \insertCite{le_FactoMineR_2008,husson_Exploratory_2017}{rpcss}.
#'
#' @param data The data as a data frame object. The data frame should possess
#'   one row per individual and columns with the individual names and multiple
#'   trait/character data.
#' @param names Name of column with the individual/genotype names as a character
#'   string.
#' @param quantitative Name of columns with the quantitative traits as a
#'   character vector.
#' @param qualitative Name of columns with the qualitative traits as a character
#'   vector.
#' @param eigen.threshold The lower limit of the eigen value of factors to be
#'   included in the estimation. The default value is the average of all the
#'   eigen values.
#' @param size The desired core set size proportion.
#' @param var.threshold The desired proportion of total variability to be
#'   sampled.
#' @param always.selected Names of genotypes to be always included in the core
#'   set as a character vector.
#'
#' @return A list of class \code{pcss.core} with the following components.
#'   \item{details}{The details of the core set generation process.}
#'   \item{raw.out}{The original output of \code{\link[FactoMineR]{PCA}},
#'   \code{\link[FactoMineR]{CA}} and \code{\link[FactoMineR]{FAMD}} functions
#'   of \code{\link[FactoMineR]{FactoMineR}}} \item{eigen}{A data frame with
#'   eigen values and their partial and cumulative contribution to percentage of
#'   variance.} \item{eigen.threshold}{The threshold eigen value used.}
#'   \item{rotation}{A matrix of rotation values or loadings.} \item{scores}{A
#'   matrix of scores from PCA, CA or FAMD.} \item{variability.ret}{A data frame
#'   of individuals/genotypes ordered by variability retained.}
#'   \item{cores.info}{A data frame of core set size and percentage variability
#'   retained according to the method used.}
#'
#' @seealso \code{\link[FactoMineR]{PCA}}, \code{\link[FactoMineR]{CA}} and
#'   \code{\link[FactoMineR]{FAMD}}
#'
#' @import gslnls
#' @import mathjaxr
#' @importFrom FactoMineR PCA CA FAMD
#' @importFrom Rdpack reprompt
#' @importFrom stats coef
#' @importFrom utils data
#' @export
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' if (requireNamespace('EvaluateCore', quietly = TRUE)) {
#'
#'   suppressPackageStartupMessages(library(EvaluateCore))
#'
#'   # Get data from EvaluateCore
#'
#'   data("cassava_EC", package = "EvaluateCore")
#'   data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
#'   quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
#'              "AVPW", "ARSR", "SRDM")
#'   qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'             "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'             "PSTR")
#'   rownames(data) <- NULL
#'
#'   # Convert qualitative data columns to factor
#'   data[, qual] <- lapply(data[, qual], as.factor)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Get core sets with PCSS (quantitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out1 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = NULL, eigen.threshold = NULL, size = 0.2,
#'                     var.threshold = 0.75)
#'
#'   out1
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
#'                     qualitative = qual, eigen.threshold = NULL,
#'                     size = 0.2, var.threshold = 0.75)
#'
#'   out2
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (quantitative and qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out3 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = qual, eigen.threshold = NULL)
#'
#'   out3
#'
#' } else {
#'   message('Package "EvaluateCore" is required to run these examples.')
#' }
#'
pcss.core <- function(data, names, quantitative, qualitative,
                      eigen.threshold = NULL,
                      size = 0.2, var.threshold = 0.75,
                      always.selected = NULL) {

  # Checks ----

  if (missing(quantitative)) {
    quantitative <- NULL
  }

  if (missing(qualitative)) {
    qualitative <- NULL
  }

  if (length(c(quantitative, qualitative)) == 1) {
    stop("Only one trait specified.")
  }

  # check if 'data' is a data frame object
  if (!is.data.frame(data)) {
    stop('"data" should be a data frame object.')
  }

  if (any(c("tbl_dataf", "tbl") %in% class(data))) {
    warning('"data" is of type tibble\nCoercing to data frame.')
    data <- as.data.frame(data)
  }

  # check if 'names' argument is character vector of unit length
  if (!(is.character(names) && length(names) == 1)) {
    stop('"names" should be a character vector of unit length.')
  }

  # check if 'quantitative' argument is a character vector
  if (!is.null(quantitative)) {
    if (!is.character(quantitative)) {
      stop('"quantitative" should be a character vector.')
    }
  }

  # check if 'qualitative' argument is a character vector
  if (!is.null(qualitative)) {
    if (!is.character(qualitative)) {
      stop('"qualitative" should be a character vector.')
    }
  }

  # check if 'names' column is present in 'data'
  if (!(names %in% colnames(data))) {
    stop(paste('Column ', names,
               ' specified as the "names" column is not present in "data".',
               sep = ""))
  }

  # check if 'quantitative' columns are present in 'data'
  if (!is.null(quantitative)) {
    if (FALSE %in% (quantitative %in% colnames(data)))  {
      stop(paste('The following column(s) specified in "quantitative" ',
                 'are not present in "data":\n',
                 paste(quantitative[!(quantitative %in% colnames(data))],
                       collapse = ", "),
                 sep = ""))
    }
  }

  # check if 'qualitative' columns are present in 'data'
  if (!is.null(qualitative)) {
    if (FALSE %in% (qualitative %in% colnames(data)))  {
      stop(paste('The following column(s) specified in "qualitative" ',
                 'are not present in "data":\n',
                 paste(qualitative[!(qualitative %in% colnames(data))],
                       collapse = ", "),
                 sep = ""))
    }
  }

  # check if overlap exists between 'quantitative' and 'qualitative'
  if ((!is.null(quantitative)) && (!is.null(qualitative))) {
    if (length(intersect(quantitative, qualitative)) != 0) {
      stop(paste('The following column(s) is/are specified in both ',
                 '"quantitative" and "qualitative":\n',
                 paste(intersect(quantitative, qualitative),
                       collapse = ", "),
                 sep = ""))
    }
  }

  # check if 'names' column is of type character
  if (!is.character(data[, names])) {
    stop('"names" column in "data" should be of type character.')
  }


  # check if 'quantitative' columns are of type numeric/integer
  if (!is.null(quantitative)) {
    intquantcols <-
      unlist(lapply(data[, quantitative],
                    function(x) FALSE %in% (is.vector(x, mode = "integer") |
                                              is.vector(x, mode = "numeric"))))
    if (TRUE %in% intquantcols) {
      stop(paste('The following "quantitative" column(s) in "data" are not ',
                 'of type numeric:\n',
                 paste(names(intquantcols[intquantcols]), collapse = ", ")))
    }
  }

  # check if 'qualitative' columns are of type factor
  if (!is.null(qualitative)) {
    intqualcols <- unlist(lapply(data[, qualitative],
                                 function(x) is.factor(x)))
    if (FALSE %in% intqualcols) {
      stop(paste('The following "qualitative" column(s) in "data" are not ',
                 'of type factor:\n',
                 paste(names(intqualcols[!intqualcols]), collapse = ", ")))
    }
  }

  # check for missing values
  missvcols <- unlist(lapply(data[, quantitative],
                             function(x) TRUE %in% is.na(x)))
  if (TRUE %in% missvcols) {
    stop(paste('The following column(s) in "data" have missing values:\n',
               paste(names(missvcols[missvcols]), collapse = ", ")))
  }

  # check for duplication in names
  if (any(duplicated(data[, names]))) {
    stop('Duplicated entries exist in "names" column.')
  }

  # check if 'eigen.threshold' argument is numeric vector of unit length
  if (!is.null(eigen.threshold)) {
    if (!(is.numeric(eigen.threshold) && length(eigen.threshold) == 1)) {
      stop('"eigen.threshold" should be a numeric vector of unit length.')
    }
  }

  # check if 'size' argument is numeric vector of unit length
  if (!is.null(size)) {
    if (!(is.numeric(size) && length(size) == 1)) {
      stop('"size" should be a numeric vector of unit length.')
    }
  }

  # check if 'size' is a proportion between 0 and 1
  if (size <= 0 || size >= 1) {
    stop('"size" should be a proportion between 0 and 1.')
  }

  # check if 'var.threshold' argument is numeric vector of unit length
  if (!is.null(var.threshold)) {
    if (!(is.numeric(var.threshold) && length(var.threshold) == 1)) {
      stop('"var.threshold" should be a numeric vector of unit length.')
    }
  }

  # check if 'var.threshold' is a proportion between 0 and 1
  if (var.threshold <= 0 || var.threshold >= 1) {
    stop('"var.threshold" should be a proportion between 0 and 1.')
  }

   if (!is.null(always.selected)) {
     # check if 'always.selected' is a character vector
    if (!is.character(always.selected)) {
      stop('"always.selected" should be a character vector.')
    }

     # check if always.selected is present in the entire set
     if (any(!(always.selected %in% data[, names]))) {
       alsel_miss <- always.selected[!(always.selected %in% data[, names])]
       stop(paste('The following accession(s) specified in "always.selected" ',
                  'are not present in "data":\n',
                  paste(alsel_miss, collapse = ", "),
                  sep = ""))
     }
  }



  # Prepare data ----

  dataf <- data[, c(names, quantitative, qualitative)]
  rownames(dataf) <- dataf[, names]
  dataf[, names] <- NULL

  pca_out <- NULL
  mca_out <- NULL
  famd_out <- NULL

  method <- NULL

  # PCA ----

  if (is.null(qualitative) && !is.null(quantitative)) {

    dataf <- dataf[, quantitative]

    dataf[, quantitative] <- lapply(dataf[, quantitative], function(x) {
      as.numeric(x)
    })

    ## Run PCA ----
    pca_out <- FactoMineR::PCA(X = dataf, scale.unit = TRUE,
                               ncp = length(quantitative),
                               graph = FALSE)

    ## Get Eigen values ----
    eig <- pca_out$eig[, "eigenvalue"]

    # round(sum(eig)) == length(quantitative)

    ## Get Loadings ----
    rot <- pca_out$svd$V

    ## Get Importance of factors/principal coordinates ----
    imp <-
      data.frame(# `Standard deviation` = pca_out$svd$vs,
        `Eigen value` = pca_out$eig[, "eigenvalue"],
        `Percentage of variance` = pca_out$eig[, "percentage of variance"],
        `Cumulative percentage of variance` =
          pca_out$eig[, "cumulative percentage of variance"],
        check.names = FALSE)

    rownames(imp) <- gsub("comp", "Dim", rownames(imp))

    ## Get Principal component scores ----
    scores <- pca_out$ind$coord

    ## Method ----
    method <- "PCA"
  }

  # Run MCA ----

  if (!is.null(qualitative) && is.null(quantitative)) {

    dataf <- dataf[, qualitative]

    # dataf[, qualitative] <- lapply(dataf[, qualitative], function(x) {
    #   as.numeric(as.factor(x))
    # })

    # dataf[, qualitative] <- lapply(dataf[, qualitative], function(x) {
    #   as.factor(x)
    # })

    ncp <- sum(unlist(lapply(dataf, function(x) length(levels(x)))))

    ## Run MCA ----
    mca_out <- FactoMineR::MCA(X = dataf,
                               ncp = ncp,
                               graph = FALSE)

    ## Get Eigen values ----
    eig <- mca_out$eig[, "eigenvalue"]

    ## Get Loadings ----
    rot <- mca_out$svd$V

    ## Get Importance of factors/principal coordinates ----
    imp <-
      data.frame(# `Standard deviation` =
                 #   mca_out$svd$vs[1:(length(qualitative) - 1)],
        `Eigen value` = mca_out$eig[, "eigenvalue"],
        `Percentage of variance` = mca_out$eig[, "percentage of variance"],
        `Cumulative percentage of variance` =
          mca_out$eig[, "cumulative percentage of variance"],
        check.names = FALSE)

    rownames(imp) <- gsub("dim", "Dim", rownames(imp))

    ## Get Principal component scores ----
    scores <- mca_out$ind$coord

    ## Method ----
    method <- "MCA"

  }

  # Run FAMD ----

  if (!is.null(qualitative) && !is.null(quantitative)) {

    dataf[, quantitative] <- lapply(dataf[, quantitative], function(x) {
      as.numeric(x)
    })

    dataf[, qualitative] <- lapply(dataf[, qualitative], function(x) {
      as.factor(x)
    })

    ## Run FAMD ----
    famd_out <- FactoMineR::FAMD(base = dataf,
                                 ncp = length(c(quantitative, qualitative)),
                                 graph = FALSE)

    ## Get Eigen values ----
    eig <- famd_out$eig[, "eigenvalue"]

    # round(sum(eig)) == length(quantitative)

    ## Get Loadings ----
    rot <- famd_out$svd$V

    ## Get Importance of factors/principal coordinates ----
    imp <-
      data.frame(#`Standard deviation` = famd_out$svd$vs,
        `Eigen value` = famd_out$eig[, "eigenvalue"],
        `Percentage of variance` = famd_out$eig[, "percentage of variance"],
        `Cumulative percentage of variance` =
          famd_out$eig[, "cumulative percentage of variance"],
        check.names = FALSE)

    rownames(imp) <- gsub("comp", "Dim", rownames(imp))

    ## Get Principal component scores ----
    scores <- famd_out$ind$coord

    ## Method ----
    method <- "FAMD"
  }

  # Contribution of individuals/genotypes to total GSS ----

  if (is.null(eigen.threshold)) {
    eigen.threshold <- mean(eig)
  } else {
    if (!(any(eig >= eigen.threshold))) {
      eigen.threshold <- mean(eig)
      warning('There are no eigen values \u2265 "eigen.threshold".\n',
              'Using average of the eigen values (', round(eigen.threshold, 2),
              ') as "eigen.threshold"', sep = "")
    }
  }


  N <- nrow(scores)
  K <- max(which((eig > eigen.threshold)))

  GSS <- N * K

  Pi <- rowSums(scores[, 1:K] ^ 2)
  Pimax <- rowSums(scores ^ 2)

  CRi <- Pi / GSS
  CRimax <- Pimax / GSS

  CRi <- sort(CRi, decreasing = TRUE)
  CRimax <- sort(CRimax, decreasing = TRUE)

  cumCRi <- cumsum(CRi)
  # cumCRimax <- cumsum(CRimax)

  # plot(cumCRimax, col = "green")
  # points(cumCRi, col = "red")

  # Select the core collection ----

  # Generalized sum of squares
  gssdf <- data.frame(Rank = seq_along(cumCRi),
                      CRi = CRi,
                      VarRet = (cumCRi / max(cumCRi)) * 100)
  gssdf <- cbind(Id = rownames(gssdf), gssdf)
  rownames(gssdf) <- NULL

  ## By size specified ----
  size.sel <- ceiling(size * N)
  size.var <- gssdf[gssdf$Rank == size.sel, ]$VarRet

  if (!is.null(always.selected)) {
    incl_ind <- !(always.selected %in% data[(size.sel + 1):nrow(data), names])
    if (any(!incl_ind)) {

      # selected excluding always.sel
      sel.gssdf <- gssdf[1:size.sel, ]
      sel.gssdf <- sel.gssdf[!(sel.gssdf$Id %in% always.selected), ]

      # always.sel
      alsel.gssdf <- gssdf[gssdf$Id %in% always.selected, ]

      # remove from sel to accomodate always.sel
      n.rem <- (nrow(sel.gssdf) + nrow(alsel.gssdf)) - size.sel

      sel.gssdf <- sel.gssdf[1:(nrow(sel.gssdf) - n.rem), ]

      # New sel
      sel.gssdf <- rbind(sel.gssdf, alsel.gssdf)
      sel.gssdf <- sel.gssdf[order(sel.gssdf$Rank), ]
      sel.gssdf$VarRet2 <- (cumsum(sel.gssdf$CRi) / max(cumCRi)) * 100

      size.var <- sel.gssdf[size.sel, ]$VarRet2

    }
  }

  ## By threshold variance ----
  var.threshold <- var.threshold * 100
  var.sel <- max(which(gssdf$VarRet <= var.threshold))

  ## With logistic regression ----

  # Fit a logistic model
  y <- gssdf$VarRet
  starta <- y[1] / (1 - y[1])
  startb <- -0.5

  maxiter <- 1024
  warnOnly <- TRUE

  dat <- data.frame(n = gssdf$Rank, y = gssdf$VarRet)

  mod <-
    gslnls::gsl_nls(
      y ~ 100 / (1 + exp(a + (b * n))),
      data = dat,
      algorithm = "lm",
      start = list(a = starta, b = startb),
      control = list(maxiter = maxiter, warnOnly = warnOnly,
                     scale = "levenberg")
    )

  # dat$pred <- 100 / (1 + exp(coef(mod)["a"] + (coef(mod)["b"] * dat$n)))

  # Compute rate of increase in variability retained
  b <- stats::coef(mod)["b"]
  dat$rate <- -b * dat$y * (100 - dat$y)

  reg.sel <- dat[dat$rate == max(dat$rate), ]$n
  reg.var <- gssdf[gssdf$Rank == reg.sel, ]$VarRet

  if (!is.null(always.selected)) {
    incl_ind <- !(always.selected %in% data[(reg.sel + 1):nrow(data), names])
    if (any(!incl_ind)) {

      # selected excluding always.sel
      sel.gssdf <- gssdf[1:reg.sel, ]

      # always.sel
      alsel.gssdf <- gssdf[gssdf$Id %in% always.selected[!incl_ind], ]

      # New sel
      sel.gssdf <- rbind(sel.gssdf, alsel.gssdf)
      sel.gssdf <- sel.gssdf[order(sel.gssdf$Rank), ]
      sel.gssdf$VarRet2 <- (cumsum(sel.gssdf$CRi) / max(cumCRi)) * 100

      reg.sel <- nrow(sel.gssdf)
      reg.var <- sel.gssdf[reg.sel, ]$VarRet2

    }
  }

  # Generate output ----

  rawout_ind <- c(pca_out = !is.null(pca_out),
                  mca_out = !is.null(mca_out),
                  famd_out = !is.null(famd_out))

  detailsdf <-
    data.frame(`Total number of individuals/genotypes` = N,
               `Quantitative traits` =  paste(quantitative, collapse = ", "),
               `Qualitative traits` =  paste(qualitative, collapse = ", "),
               `Method` = method,
               `Threshold eigen value` = eigen.threshold,
               `Number of eigen values selected` = K,
               `Threshold size` = size,
               `Threshold variance (%)` = var.threshold,
               check.names = FALSE)

  detailsdf <- t(detailsdf)
  detailsdf <- cbind(Detail = rownames(detailsdf),
                     Value = detailsdf[, 1])
  detailsdf <- data.frame(detailsdf, check.names = FALSE)

  rownames(detailsdf) <- NULL

  coreinfodf <- data.frame(Method = c("By size specified",
                                      "By threshold variance",
                                      "By logistic regression"),
                           Size = c(size.sel, var.sel, reg.sel),
                           VarRet = c(size.var, var.threshold, reg.var))

  rownames(coreinfodf) <- NULL

  out <- list(details = detailsdf,
              raw.out = get(names(which(rawout_ind))),
              eigen = imp,
              eigen.threshold = eigen.threshold,
              rotation = rot,
              scores = scores,
              variability.ret = gssdf,
              cores.info = coreinfodf,
              always.selected = always.selected)

  quali.levels <- NULL

  if (!is.null(qualitative)) {
    quali.levels <-  lapply(dataf[, qualitative],
                            function(x) levels(x))
  }

  class(out) <- "pcss.core"

  attr(x = out, which = "method") <- method
  attr(x = out, which = "quant") <- quantitative
  attr(x = out, which = "quali") <- qualitative
  attr(x = out, which = "quali.levels") <- quali.levels
  attr(x = out, which = "slope") <- b

  return(out)

}

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Generic function for \code{coreplot.pcss.core}
#'
#' @param x An object of class \code{pcss.core}.
#'
#' @param ...  Unused.
#'
#' @export
#' @keywords internal
#'
coreplot <- function(x, ...) {
  UseMethod("coreplot")
}

#' Generic function for \code{contrib.pcss.core}
#'
#' @param x An object of class \code{pcss.core}.
#'
#' @param ...  Unused.
#'
#' @export
#' @keywords internal
#'
contrib <- function(x, ...) {
  UseMethod("contrib")
}

### This file is part of 'rpcss' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# rpcss is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# rpcss is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Generate Biplots from \code{pcss.core} Output
#'
#' \code{biplot.pcss.core} generates biplots of scores of genotypes with or
#' without vectors for traits from the output of \code{pcss.core}.
#'
#' Use \code{"size"} to highlight core collection according to the threshold
#' \code{size} criterion or use \code{"variance"} to highlight core collection
#' according to the variability threshold criterion or use  \code{"logistic"} to
#' highlight core collection generated according to inflection point of rate of
#' progress of cumulative variability retained identified by logistic
#' regression. Use \code{"none"} to not highlight any accessions.
#'
#' @param x An object of class \code{pcss.core}.
#' @param ndim The number of dimensions for which biplots have to plotted.
#' @param highlight.core The core collection to be highlighted. Either
#'   \code{"size"}, \code{"variance"}, \code{"logistic"}, or \code{"none"}. See
#'   \strong{Details}.
#' @param show.traits Which kind of the traits to be shown in the biplot. Either
#'   \code{"all"}, \code{"none"}, \code{"quantitative"} or \code{"qualitative"}.
#' @param qual.scale A scale factor to be applied to qualitative trait
#'   coordinates plotted in biplot.
#' @param quant.scale A scale factor to be applied to quantitative trait
#'   coordinates plotted in biplot.
#' @param point.alpha Alpha transparency value for biplot points.
#' @param segment.alpha Alpha transparency value for biplot segments.
#' @param ... Unused.
#'
#' @return A list of biplots as \code{ggplot} objects.
#'
#' @seealso \code{\link[rpcss]{pcss.core}}, \code{\link[FactoMineR]{plot.PCA}},
#'   \code{\link[FactoMineR]{plot.MCA}}, \code{\link[FactoMineR]{plot.FAMD}},
#'   \code{\link[factoextra]{fviz_pca}}, \code{\link[factoextra]{fviz_mca}},
#'   \code{\link[factoextra]{fviz_famd}}
#'
#' @import ggplot2
#' @importFrom dplyr bind_rows
#' @importFrom ggrepel geom_label_repel
#' @importFrom stats biplot
#' @importFrom utils combn
#' @import ggplot2
#' @exportS3Method rpcss::biplot
#'
#' @examples
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' if (requireNamespace('EvaluateCore', quietly = TRUE) &
#'     requireNamespace("factoextra", quietly = TRUE)) {
#'
#'   suppressPackageStartupMessages(library(EvaluateCore))
#'
#'   # Get data from EvaluateCore
#'
#'   data("cassava_EC", package = "EvaluateCore")
#'   data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
#'   quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
#'              "AVPW", "ARSR", "SRDM")
#'   qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'             "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'             "PSTR")
#'   rownames(data) <- NULL
#'
#'   # Convert qualitative data columns to factor
#'   data[, qual] <- lapply(data[, qual], as.factor)
#'
#'
#'   library(FactoMineR)
#'   suppressPackageStartupMessages(library(factoextra))
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # With quantitative data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out1 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = NULL, eigen.threshold = NULL, size = 0.2,
#'                     var.threshold = 0.75)
#'
#'   # Plot biplot
#'   biplot(out1, ndim = 3, highlight.core = "size", quant.scale = 3,
#'          point.alpha = 0.5)
#'
#'   # Plot biplot with FactoMineR
#'   plot(out1$raw.out, choix=c("ind"), label  = "none", axes = c(1, 2))
#'
#'   plot(out1$raw.out, choix=c("ind"), label  = "none", axes = c(1, 3))
#'
#'   plot(out1$raw.out, choix=c("ind"), label  = "none", axes = c(2, 3))
#'
#'   # Plot biplot with factoextra
#'   fviz_pca_biplot(out1$raw.out, geom.ind = "point", axes = c(1, 2))
#'
#'   fviz_pca_biplot(out1$raw.out, geom.ind = "point", axes = c(1, 3))
#'
#'   fviz_pca_biplot(out1$raw.out, geom.ind = "point", axes = c(2, 3))
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
#'                     qualitative = qual, eigen.threshold = NULL,
#'                     size = 0.2, var.threshold = 0.75)
#'
#'   # Plot biplot
#'   biplot(out2, ndim = 3, highlight.core = "size", qual.scale = 1,
#'          point.alpha = 0.5)
#'
#'   # Plot biplot with FactoMineR
#'   plot(out2$raw.out, choix=c("ind"), label  = "none", axes = c(1, 2))
#'
#'   plot(out2$raw.out, choix=c("ind"), label  = "none", axes = c(1, 3))
#'
#'   plot(out2$raw.out, choix=c("ind"), label  = "none", axes = c(2, 3))
#'
#'   # Plot biplot with factoextra
#'   fviz_mca_biplot(out2$raw.out, geom.ind = "point", axes = c(1, 2))
#'
#'   fviz_mca_biplot(out2$raw.out, geom.ind = "point", axes = c(1, 3))
#'
#'   fviz_mca_biplot(out2$raw.out, geom.ind = "point", axes = c(2, 3))
#'
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Get core sets with PCSS (quantitative and qualitative data)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   out3 <- pcss.core(data = data, names = "Genotypes",
#'                     quantitative = quant,
#'                     qualitative = qual, eigen.threshold = NULL)
#'
#'   # Plot biplot
#'   biplot(out3, ndim = 3, highlight.core = "size",
#'          quant.scale = 3, qual.scale = 1,
#'          point.alpha = 0.5)
#'
#'   # Plot biplot with FactoMineR
#'   plot(out3$raw.out, choix=c("ind"), label  = "none", axes = c(1, 2))
#'
#'   plot(out3$raw.out, choix=c("ind"), label  = "none", axes = c(1, 3))
#'
#'   plot(out3$raw.out, choix=c("ind"), label  = "none", axes = c(2, 3))
#'
#'   # Plot biplot with factoextra
#'
#'   # Fix rownames
#'   row.names(out3$raw.out$quali.var$coord) <-
#'     unlist(lapply(seq_along(data[, qual]),
#'                   function(i) paste(qual[i],
#'                                     levels(data[, qual[i]]), sep = "_")))
#'
#'   fviz_famd_ind(out3$raw.out, geom = "point", axes = c(1, 2))
#'
#'   fviz_famd_ind(out3$raw.out, geom = "point", axes = c(1, 3))
#'
#'   fviz_famd_ind(out3$raw.out, geom = "point", axes = c(2, 3))
#'
#' } else {
#'   message('Packages "EvaluateCore" and "factoextra" are required to run these examples.')
#' }
#'
biplot.pcss.core <- function(x,
                             ndim = 3, # at least 2
                             highlight.core = c("size", "variance",
                                                "logistic", "none"),
                             show.traits = c("all", "none",
                                             "quantitative", "qualitative"),
                             qual.scale = 1,
                             quant.scale = 1,
                             point.alpha = 0.8,
                             segment.alpha = 0.8, ...) {

  # Checks ----

  # Check class of "x"
  if (!is(x, "pcss.core")) {
    stop('"x" is not of class "pcss.core".')
  }

  # check if 'ndim' argument is integer vector of unit length
  if (!(as.integer(ndim) == ndim && length(ndim) == 1)) {
    stop('"ndim" should be a integer vector of unit length.')
  }

  # check if at least two dimensions are plotted in ndim
  if (ndim < 2) {
    stop('At least 2 dimensions are to be specified in "ndim".')
  }

  method <- attr(x, "method")

  # check if ndim is not greater than total dimensions
  total.ndim <- ncol(x$raw.out$ind$coord)

  if (ndim > total.ndim) {
    warning('"ndim" is greater than the total number of dimensions.\n',
            paste('Using the total number of dimensions (',
                  total.ndim, ') as "ndim".', sep = ""))
    ndim <- total.ndim
  }

  # check show.traits argument
  show.traits <- match.arg(show.traits)

  if (method == "MCA" || method == "FAMD") {
    # check qual.scale argument is numeric vector of unit length
    if (!(is.numeric(qual.scale) && length(qual.scale) == 1)) {
      stop('"qual.scale" should be a numeric vector of unit length.')
    }
  }

  if (method == "PCA" || method == "FAMD") {
    # check quant.scale argument is numeric vector of unit length
    if (!(is.numeric(quant.scale) && length(quant.scale) == 1)) {
      stop('"quanti.scale" should be a numeric vector of unit length.')
    }
  }

  # check point.alpha argument is numeric vector of unit length
  if (!(is.numeric(point.alpha) && length(point.alpha) == 1)) {
    stop('"point.alpha" should be a numeric vector of unit length.')
  }

  # check if 'point.alpha' is a value between 0 and 1
  if (point.alpha <= 0 || point.alpha >= 1) {
    stop('"point.alpha" should be a value between 0 and 1.')
  }

  # check segment.alpha argument is numeric vector of unit length
  if (!(is.numeric(segment.alpha) && length(segment.alpha) == 1)) {
    stop('"segment.alpha" should be a numeric vector of unit length.')
  }

  # check if 'segment.alpha' is a value between 0 and 1
  if (segment.alpha <= 0 || segment.alpha >= 1) {
    stop('"segment.alpha" should be a value between 0 and 1.')
  }

  # Check 'highlight.core'
  highlight.core <- match.arg(highlight.core)

  # Get coordinates for biplot ----

  if (method == "PCA") {

    pca_out <- x$raw.out

    ## Coordinates for biplot
    ind_coord <- pca_out$ind$coord
    ind_coord <- ind_coord[, 1:ndim]
    colnames(ind_coord) <- gsub("Dim.", "Dim ", colnames(ind_coord))

    quant_coord <- pca_out$var$coord
    quant_coord <- quant_coord[, 1:ndim]
    colnames(quant_coord) <- gsub("Dim.", "Dim ", colnames(quant_coord))
    quant_coord <- quant_coord * quant.scale
  }

  if (method == "MCA") {

    mca_out <- x$raw.out

    ## Coordinates for biplot
    ind_coord <- mca_out$ind$coord
    ind_coord <- ind_coord[, 1:ndim]
    colnames(ind_coord) <- gsub("Dim.", "Dim ", colnames(ind_coord))

    qual_coord <- mca_out$var$coord
    qual_coord <- qual_coord[, 1:ndim]
    colnames(qual_coord) <- gsub("Dim.", "Dim ", colnames(qual_coord))
    qual_coord <- qual_coord * qual.scale
  }

  if (method == "FAMD") {

    famd_out <- x$raw.out

    ## Coordinates for biplot
    ind_coord <- famd_out$ind$coord
    ind_coord <- ind_coord[, 1:ndim]
    colnames(ind_coord) <- gsub("Dim.", "Dim ", colnames(ind_coord))

    quant_coord <- famd_out$quanti.var$coord
    quant_coord <- quant_coord[, 1:ndim]
    colnames(quant_coord) <- gsub("Dim.", "Dim ", colnames(quant_coord))
    quant_coord <- quant_coord * quant.scale

    qual_coord <- famd_out$quali.var$coord
    qual_coord <- qual_coord[, 1:ndim]
    colnames(qual_coord) <- gsub("Dim.", "Dim ", colnames(qual_coord))
    qual_coord <- qual_coord * qual.scale

    quali.levels <- attr(x, "quali.levels")
    qual_levels <-
      lapply(seq_along(quali.levels), function(i) {
        data.frame(qual = names(quali.levels)[i],
                   qual_levels = quali.levels[[i]])
      })
    qual_levels <- dplyr::bind_rows(qual_levels)

    if (any(qual_levels$qual_levels != rownames(qual_coord))) {
      warning('Mismatch in levels of qualitative traits and ',
              'the names of qualitative trait level coordinates.')
    } else {
      rownames(qual_coord) <- paste(qual_levels$qual,
                                    qual_levels$qual_levels, sep = "_")
    }
  }

  if (highlight.core != "none") {
    core <- subset(x = x, criterion = highlight.core)

    ind_coord <- data.frame(ind_coord, check.names = FALSE)

    ind_coord$core <- rownames(ind_coord) %in% core

    ind_coord$core <- ifelse(ind_coord$core == "TRUE", "red", "black")

  } else {

    ind_coord <- data.frame(ind_coord, check.names = FALSE)
  }

  # Plot Biplot ----

  imp <- x$eigen

  biplot_comb <- data.frame(t(combn(x = setdiff(colnames(ind_coord), "core"),
                                    m = 2)))
  biplot_comb$label <- paste(biplot_comb$X1, biplot_comb$X2, sep = " vs. ")

  biplot_list <- vector("list", length = nrow(biplot_comb))
  names(biplot_list) <- biplot_comb$label

  for (i in seq_along(biplot_list)) {

    xlb <- paste(biplot_comb[i, 1], " (",
                 round(imp[biplot_comb[i, 1], ]$`Percentage of variance`, 2),
                 "% explained variance)", sep = "")
    ylb <- paste(biplot_comb[i, 2], " (",
                 round(imp[biplot_comb[i, 2], ]$`Percentage of variance`, 2),
                 "% explained variance)", sep = "")

    if (highlight.core != "none") {

      bipg <- ggplot(data = ind_coord,
                     aes(x = .data[[biplot_comb[i, 1]]],
                         y = .data[[biplot_comb[i, 2]]])) +
        geom_vline(xintercept = 0, linetype = 2, colour = "gray20") +
        geom_hline(yintercept = 0, linetype = 2, colour = "gray20") +
        geom_point(alpha = point.alpha, show.legend = FALSE,
                   colour = ind_coord$core) +
        xlab(label = xlb) +
        ylab(label = ylb) +
        theme_bw()

    } else {

      bipg <- ggplot(data = ind_coord,
                     aes(x = .data[[biplot_comb[i, 1]]],
                         y = .data[[biplot_comb[i, 2]]])) +
        geom_vline(xintercept = 0, linetype = 2, colour = "gray20") +
        geom_hline(yintercept = 0, linetype = 2, colour = "gray20") +
        geom_point(alpha = point.alpha) +
        xlab(label = xlb) +
        ylab(label = ylb) +
        theme_bw()

    }

    method <- attributes(x)$method

    if ((method == "PCA" &&
         (show.traits == "all" || show.traits == "quantitative")) ||
        method == "FAMD" && show.traits == "quantitative") {

      bipg <- bipg +
        geom_segment(data = quant_coord,
                     aes(x = 0, y = 0,
                         xend = .data[[biplot_comb[i, 1]]],
                         yend = .data[[biplot_comb[i, 2]]]),
                     arrow = arrow(length = unit(0.2, "cm")),
                     color = "#619CFF", alpha = segment.alpha) +
        geom_label_repel(data = quant_coord,
                         aes(x = .data[[biplot_comb[i, 1]]],
                             y = .data[[biplot_comb[i, 2]]],
                             label = rownames(quant_coord)),
                         # vjust = -0.5,
                         fill = "#619CFF", colour = "white",
                         alpha = segment.alpha)
    }

    if ((method == "MCA" &&
         (show.traits == "all" || show.traits == "qualitative")) ||
        method == "FAMD" && show.traits == "qualitative") {

      bipg <- bipg +
        geom_segment(data = qual_coord,
                     aes(x = 0, y = 0,
                         xend = .data[[biplot_comb[i, 1]]],
                         yend = .data[[biplot_comb[i, 2]]]),
                     arrow = arrow(length = unit(0.2, "cm")),
                     color = "#00BA38", alpha = segment.alpha) +
        geom_label_repel(data = qual_coord,
                         aes(x = .data[[biplot_comb[i, 1]]],
                             y = .data[[biplot_comb[i, 2]]],
                             label = rownames(qual_coord)),
                         # vjust = -0.5,
                         fill = "#00BA38", colour = "white",
                         alpha = segment.alpha)
    }

    if (method == "FAMD" && show.traits == "all") {

      trait_coord <- rbind(cbind(data.frame(quant_coord, check.names = FALSE),
                                 Type = "Quantitative"),
                           cbind(data.frame(qual_coord, check.names = FALSE),
                                 Type = "Qualitative"))

      bipg <- bipg +
        geom_segment(data = trait_coord,
                     aes(x = 0, y = 0,
                         xend = .data[[biplot_comb[i, 1]]],
                         yend = .data[[biplot_comb[i, 2]]],
                         color = Type),
                     arrow = arrow(length = unit(0.2, "cm")),
                     alpha = segment.alpha, show.legend = FALSE) +
        geom_label_repel(data = trait_coord,
                         aes(x = .data[[biplot_comb[i, 1]]],
                             y = .data[[biplot_comb[i, 2]]],
                             label = rownames(trait_coord),
                             fill = Type),
                         # vjust = -0.5,
                         colour = "white",
                         alpha = segment.alpha, show.legend = FALSE) +
        scale_fill_manual(values = c("#00BA38", "#619CFF")) +
        scale_colour_manual(values = c("#00BA38", "#619CFF"))
    }


    biplot_list[[i]] <- bipg

    rm(bipg, xlb, ylb)

  }

  # patchwork::wrap_plots(biplot_list)

  return(biplot_list)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biplot.pcss.core.R
\name{biplot.pcss.core}
\alias{biplot.pcss.core}
\title{Generate Biplots from \code{pcss.core} Output}
\usage{
\method{biplot}{pcss.core}(
  x,
  ndim = 3,
  highlight.core = c("size", "variance", "logistic", "none"),
  show.traits = c("all", "none", "quantitative", "qualitative"),
  qual.scale = 1,
  quant.scale = 1,
  point.alpha = 0.8,
  segment.alpha = 0.8,
  ...
)
}
\arguments{
\item{x}{An object of class \code{pcss.core}.}

\item{ndim}{The number of dimensions for which biplots have to plotted.}

\item{highlight.core}{The core collection to be highlighted. Either
\code{"size"}, \code{"variance"}, \code{"logistic"}, or \code{"none"}. See
\strong{Details}.}

\item{show.traits}{Which kind of the traits to be shown in the biplot. Either
\code{"all"}, \code{"none"}, \code{"quantitative"} or \code{"qualitative"}.}

\item{qual.scale}{A scale factor to be applied to qualitative trait
coordinates plotted in biplot.}

\item{quant.scale}{A scale factor to be applied to quantitative trait
coordinates plotted in biplot.}

\item{point.alpha}{Alpha transparency value for biplot points.}

\item{segment.alpha}{Alpha transparency value for biplot segments.}

\item{...}{Unused.}
}
\value{
A list of biplots as \code{ggplot} objects.
}
\description{
\code{biplot.pcss.core} generates biplots of scores of genotypes with or
without vectors for traits from the output of \code{pcss.core}.
}
\details{
Use \code{"size"} to highlight core collection according to the threshold
\code{size} criterion or use \code{"variance"} to highlight core collection
according to the variability threshold criterion or use  \code{"logistic"} to
highlight core collection generated according to inflection point of rate of
progress of cumulative variability retained identified by logistic
regression. Use \code{"none"} to not highlight any accessions.
}
\examples{

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if (requireNamespace('EvaluateCore', quietly = TRUE) &
    requireNamespace("factoextra", quietly = TRUE)) {

  suppressPackageStartupMessages(library(EvaluateCore))

  # Get data from EvaluateCore

  data("cassava_EC", package = "EvaluateCore")
  data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
  quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
             "AVPW", "ARSR", "SRDM")
  qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
            "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
            "PSTR")
  rownames(data) <- NULL

  # Convert qualitative data columns to factor
  data[, qual] <- lapply(data[, qual], as.factor)


  library(FactoMineR)
  suppressPackageStartupMessages(library(factoextra))

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# With quantitative data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out1 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = NULL, eigen.threshold = NULL, size = 0.2,
                    var.threshold = 0.75)

  # Plot biplot
  biplot(out1, ndim = 3, highlight.core = "size", quant.scale = 3,
         point.alpha = 0.5)

  # Plot biplot with FactoMineR
  plot(out1$raw.out, choix=c("ind"), label  = "none", axes = c(1, 2))

  plot(out1$raw.out, choix=c("ind"), label  = "none", axes = c(1, 3))

  plot(out1$raw.out, choix=c("ind"), label  = "none", axes = c(2, 3))

  # Plot biplot with factoextra
  fviz_pca_biplot(out1$raw.out, geom.ind = "point", axes = c(1, 2))

  fviz_pca_biplot(out1$raw.out, geom.ind = "point", axes = c(1, 3))

  fviz_pca_biplot(out1$raw.out, geom.ind = "point", axes = c(2, 3))

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
                    qualitative = qual, eigen.threshold = NULL,
                    size = 0.2, var.threshold = 0.75)

  # Plot biplot
  biplot(out2, ndim = 3, highlight.core = "size", qual.scale = 1,
         point.alpha = 0.5)

  # Plot biplot with FactoMineR
  plot(out2$raw.out, choix=c("ind"), label  = "none", axes = c(1, 2))

  plot(out2$raw.out, choix=c("ind"), label  = "none", axes = c(1, 3))

  plot(out2$raw.out, choix=c("ind"), label  = "none", axes = c(2, 3))

  # Plot biplot with factoextra
  fviz_mca_biplot(out2$raw.out, geom.ind = "point", axes = c(1, 2))

  fviz_mca_biplot(out2$raw.out, geom.ind = "point", axes = c(1, 3))

  fviz_mca_biplot(out2$raw.out, geom.ind = "point", axes = c(2, 3))


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (quantitative and qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out3 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = qual, eigen.threshold = NULL)

  # Plot biplot
  biplot(out3, ndim = 3, highlight.core = "size",
         quant.scale = 3, qual.scale = 1,
         point.alpha = 0.5)

  # Plot biplot with FactoMineR
  plot(out3$raw.out, choix=c("ind"), label  = "none", axes = c(1, 2))

  plot(out3$raw.out, choix=c("ind"), label  = "none", axes = c(1, 3))

  plot(out3$raw.out, choix=c("ind"), label  = "none", axes = c(2, 3))

  # Plot biplot with factoextra

  # Fix rownames
  row.names(out3$raw.out$quali.var$coord) <-
    unlist(lapply(seq_along(data[, qual]),
                  function(i) paste(qual[i],
                                    levels(data[, qual[i]]), sep = "_")))

  fviz_famd_ind(out3$raw.out, geom = "point", axes = c(1, 2))

  fviz_famd_ind(out3$raw.out, geom = "point", axes = c(1, 3))

  fviz_famd_ind(out3$raw.out, geom = "point", axes = c(2, 3))

} else {
  message('Packages "EvaluateCore" and "factoextra" are required to run these examples.')
}

}
\seealso{
\code{\link[rpcss]{pcss.core}}, \code{\link[FactoMineR]{plot.PCA}},
  \code{\link[FactoMineR]{plot.MCA}}, \code{\link[FactoMineR]{plot.FAMD}},
  \code{\link[factoextra]{fviz_pca}}, \code{\link[factoextra]{fviz_mca}},
  \code{\link[factoextra]{fviz_famd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contrib.pcss.core.R, R/generics.R
\name{contrib}
\alias{contrib}
\alias{contrib.default}
\alias{contrib.pcss.core}
\title{Plot Contribution or Loadings of Traits for each Dimension/Factor from
\code{pcss.core} Output}
\usage{
contrib(x, ...)

\method{contrib}{default}(
  x,
  ndim = NULL,
  plot.loadings = FALSE,
  use.sign = TRUE,
  sort.value = TRUE,
  ...
)

\method{contrib}{pcss.core}(
  x,
  ndim = NULL,
  plot.loadings = FALSE,
  use.sign = TRUE,
  sort.value = TRUE,
  ...
)

contrib(x, ...)
}
\arguments{
\item{x}{An object of class \code{pcss.core}.}

\item{...}{Unused.}

\item{ndim}{The number of dimensions for which contribution or loadings of
traits are to be plotted.}

\item{plot.loadings}{If \code{TRUE}, the loadings or "right singular vectors"
are plotted instead of contributions. Default is \code{FALSE}.}

\item{use.sign}{If \code{TRUE}, contributions of variables are given the sign
of their corresponding coordinates. Default is \code{TRUE}.}

\item{sort.value}{If \code{TRUE}, the bars are sorted according to their
value.}
}
\value{
The contributions/loadings bar plot as a \code{ggplot} object.
}
\description{
\code{contrib.pcss.core} generates bar plots of contributions or loadings
("right singular vectors") of traits for each dimension/factor from the
output of \code{pcss.core}.
}
\examples{

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if (requireNamespace('EvaluateCore', quietly = TRUE) &
    requireNamespace("factoextra", quietly = TRUE)) {

  # Get data from EvaluateCore

  data("cassava_EC", package = "EvaluateCore")
  data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
  quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
             "ARSR", "SRDM")
  qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
            "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
            "PSTR")
  rownames(data) <- NULL

  # Convert qualitative data columns to factor
  data[, qual] <- lapply(data[, qual], as.factor)


  library(FactoMineR)
  suppressPackageStartupMessages(library(factoextra))

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# With quantitative data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out1 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = NULL, eigen.threshold = NULL, size = 0.2,
                    var.threshold = 0.75)

  # Plot contributions of genotypes - with sign - sorted
  contrib(x = out1, ndim = 5)

  # Plot contributions of genotypes - without sign - sorted
  contrib(x = out1, ndim = 5, use.sign = FALSE)

  # Plot loadings/coordinates of genotypes - with sign - sorted
  contrib(x = out1, ndim = 5, plot.loadings = TRUE)

  # Plot contributions of genotypes - with sign - unsorted
  contrib(x = out1, ndim = 5, sort.value = FALSE)

  # Plot biplot with factoextra
  fviz_contrib(out1$raw.out, choice = "var", axes = 1)
  fviz_contrib(out1$raw.out, choice = "var", axes = 2)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
                    qualitative = qual, eigen.threshold = NULL,
                    size = 0.2, var.threshold = 0.75)

  # Plot contributions of genotypes - with sign - sorted
  contrib(x = out2, ndim = 5)

  # Plot contributions of genotypes - without sign - sorted
  contrib(x = out2, ndim = 5, use.sign = FALSE)

  # Plot loadings/coordinates of genotypes - with sign - sorted
  contrib(x = out2, ndim = 5, plot.loadings = TRUE)

  # Plot contributions of genotypes - with sign - unsorted
  contrib(x = out2, ndim = 5, sort.value = FALSE)

  # Plot biplot with factoextra
  fviz_contrib(out2$raw.out, choice = "var", axes = 1)
  fviz_contrib(out2$raw.out, choice = "var", axes = 2)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (quantitative and qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out3 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = qual, eigen.threshold = NULL)

  # Plot contributions of genotypes - sorted
  contrib(x = out3, ndim = 5)

  # Plot contributions of genotypes - without sign - sorted
  contrib(x = out3, ndim = 5, use.sign = FALSE)

  # Plot loadings/coordinates of genotypes - sorted
  contrib(x = out3, ndim = 5, plot.loadings = TRUE)

  # Plot contributions of genotypes - with sign - unsorted
  contrib(x = out3, ndim = 5, sort.value = FALSE)

  # Plot biplot with factoextra
  # Has a bug (https://github.com/kassambara/factoextra/issues/184)
  # fviz_contrib(out3$raw.out, choice = "quanti.var", axes = 1)
  # fviz_contrib(out3$raw.out, choice = "quali.var", axes = 1)
  # fviz_contrib(out3$raw.out, choice = "quanti.var", axes = 2)
  # fviz_contrib(out3$raw.out, choice = "quali.var", axes = 2)

} else {
  message('Packages "EvaluateCore" and "factoextra" are required to run these examples.')
}

}
\seealso{
\code{\link[rpcss]{pcss.core}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coreplot.pcss.core.R, R/generics.R
\name{coreplot}
\alias{coreplot}
\alias{coreplot.default}
\alias{coreplot.pcss.core}
\title{Plot the cumulative variability retained by individuals/genotypes from
\code{pcss.core} Output}
\usage{
coreplot(x, ...)

\method{coreplot}{default}(x, criterion = c("size", "variance", "logistic"), ...)

\method{coreplot}{pcss.core}(x, criterion = c("size", "variance", "logistic"), ...)

coreplot(x, ...)
}
\arguments{
\item{x}{An object of class \code{pcss.core}.}

\item{...}{Unused.}

\item{criterion}{The core collection generation criterion. Either
\code{"size"}, \code{"variance"}, or \code{"logistic"}. See
\strong{Details}.}
}
\value{
A plot of cumulative variability retained by individuals/genotypes as
  a \code{ggplot} object. In case of \code{criterion = "logistic"}, a list
  with plots of cumulative variability retained by individuals/genotypes and
  rate of progress of cumulative contribution to variability. The size and
  variability retained by core collection are highlighted in each plot.
}
\description{
\code{coreplot.pcss.core} generates plots of cumulative variability retained
by individuals/genotypes from \code{pcss.core} Output. The size of core
collection and the corresponding cumulative variance retained are highlighted
according to the criterion used.
}
\details{
Use \code{"size"} to highlight core collection according to the threshold
\code{size} criterion or use \code{"variance"} to highlight core collection
according to the variability threshold criterion or use  \code{"logistic"} to
highlight core collection generated according to inflection point of rate of
progress of cumulative variability retained identified by logistic
regression.
}
\examples{

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if (requireNamespace('EvaluateCore', quietly = TRUE)) {

  suppressPackageStartupMessages(library(EvaluateCore))

  # Get data from EvaluateCore

  data("cassava_EC", package = "EvaluateCore")
  data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
  quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
             "AVPW", "ARSR", "SRDM")
  qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
            "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
            "PSTR")
  rownames(data) <- NULL

  # Convert qualitative data columns to factor
  data[, qual] <- lapply(data[, qual], as.factor)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# With quantitative data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out1 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = NULL, eigen.threshold = NULL, size = 0.2,
                    var.threshold = 0.75)

  # For core set constituted by size criterion
  coreplot(x = out1, criterion = "size")

  # For core set constituted by variance criterion
  coreplot(x = out1, criterion = "variance")

  # For core set constituted by logistic regression criterion
  coreplot(x = out1, criterion = "logistic")

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
                    qualitative = qual, eigen.threshold = NULL,
                    size = 0.2, var.threshold = 0.75)

  # For core set constituted by size criterion
  coreplot(x = out2, criterion = "size")

  # For core set constituted by variance criterion
  coreplot(x = out2, criterion = "variance")

  # For core set constituted by logistic regression criterion
  coreplot(x = out2, criterion = "logistic")

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (quantitative and qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out3 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = qual, eigen.threshold = NULL)

  # For core set constituted by size criterion
  coreplot(x = out3, criterion = "size")

  # For core set constituted by variance criterion
  coreplot(x = out3, criterion = "variance")

  # For core set constituted by logistic regression criterion
  coreplot(x = out3, criterion = "logistic")

} else {
  message('Package "EvaluateCore" is required to run these examples.')
}

}
\seealso{
\code{\link[rpcss]{pcss.core}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.pcss.core.R
\name{print.pcss.core}
\alias{print.pcss.core}
\title{Prints summary of \code{pcss.core} object}
\usage{
\method{print}{pcss.core}(x, ...)
}
\arguments{
\item{x}{An object of class \code{pcss.core}.}

\item{...}{Unused.}
}
\value{
The argument x, invisibly as for all \code{\link[base]{print}}
  methods.
}
\description{
\code{pcss.core} prints to console the summary of an object of class
\code{pcss.core} including the dimensionality reduction method used, the
basic details including parameters and the information on the core sets that
can be constituted.
}
\seealso{
\code{\link[rpcss]{pcss.core}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcss.core.R
\name{pcss.core}
\alias{pcss.core}
\title{Principal Component Scoring to Generate Core collections}
\usage{
pcss.core(
  data,
  names,
  quantitative,
  qualitative,
  eigen.threshold = NULL,
  size = 0.2,
  var.threshold = 0.75,
  always.selected = NULL
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the individual/genotype names as a character
string.}

\item{quantitative}{Name of columns with the quantitative traits as a
character vector.}

\item{qualitative}{Name of columns with the qualitative traits as a character
vector.}

\item{eigen.threshold}{The lower limit of the eigen value of factors to be
included in the estimation. The default value is the average of all the
eigen values.}

\item{size}{The desired core set size proportion.}

\item{var.threshold}{The desired proportion of total variability to be
sampled.}

\item{always.selected}{Names of genotypes to be always included in the core
set as a character vector.}
}
\value{
A list of class \code{pcss.core} with the following components.
  \item{details}{The details of the core set generation process.}
  \item{raw.out}{The original output of \code{\link[FactoMineR]{PCA}},
  \code{\link[FactoMineR]{CA}} and \code{\link[FactoMineR]{FAMD}} functions
  of \code{\link[FactoMineR]{FactoMineR}}} \item{eigen}{A data frame with
  eigen values and their partial and cumulative contribution to percentage of
  variance.} \item{eigen.threshold}{The threshold eigen value used.}
  \item{rotation}{A matrix of rotation values or loadings.} \item{scores}{A
  matrix of scores from PCA, CA or FAMD.} \item{variability.ret}{A data frame
  of individuals/genotypes ordered by variability retained.}
  \item{cores.info}{A data frame of core set size and percentage variability
  retained according to the method used.}
}
\description{
Generate a Core Collection with Principal Component Scoring Strategy (PCSS)
\insertCite{hamon_proposed_1990,noirot_principal_1996,noirot_method_2003}{rpcss}
using qualitative and/or quantitative trait data. \loadmathjax
}
\details{
A core collection is constituted from an entire collection of \mjseqn{N}
genotypes using quantitative data of \mjseqn{J} traits using Principal
Component Scoring Strategy (PCSS)
\insertCite{hamon_proposed_1990,noirot_principal_1996,noirot_method_2003}{rpcss}
as follows:

\enumerate{

\item Principal Component Analysis (PCA) is performed on the standardized
genotype \mjseqn{\times} trait data. This takes care of multicollinearity
between the traits to generate \mjseqn{J} standardized and independent
variables or factors or principal component.

\item Considering only a subset of factors \mjseqn{K}, the Generalized Sum of
 Squares (GSS) of N individuals in K factorial spaces is computed as
\mjseqn{N \times K}.

\mjseqn{K} can be the number of factors for which the eigen value
\mjseqn{\lambda} is greater than a threshold value such as 1 (Kaiser-Guttman
criterion) or the average of all the eigen values.

\item The contribution of the \mjseqn{i}th genotype to GSS (\mjseqn{P_{i}})
or total variability is calculated as below.

\mjsdeqn{P_{i} = \sum_{j = 1}^{K} x_{ij}^{2}}

Where \mjseqn{x_{ij}} is the component score or coordinate of the
\mjseqn{i}th genotype on the \mjseqn{j}th principal component.

\item For each genotype, its relative contribution to GSS or total
variability is computed as below.

\mjsdeqn{CR_{i} = \frac{P_{i}}{N \times K}}

\item The genotypes are sorted in descending order of magnitude of their
contribution to GSS and then the cumulative contribution of successive
genotypes to GSS is computed.

\item The core collection can then be selected by three different methods.

 \enumerate{

 \item Selection of fixed proportion or percentage or number of the top
 accessions.

 \item Selection of the top accessions that contribute up to a fixed
 percentage of the GSS.

 \item Fitting a logistic regression model of the following form to the
 cumulative contribution of successive genotypes to GSS
 \insertCite{balakrishnan_method_2000}{rpcss}.

 \mjsdeqn{\frac{y}{A-y} = e^{a + bn}}

 The above equation can  be reparameterized as below.

 \mjsdeqn{\log_{e} \left ( {\frac{y}{A-y}} \right ) = a + bn}

 Where, \mjseqn{a} and \mjseqn{b} are the intercept and regression
 coefficient, respectively; \mjseqn{y} is the cumulative contribution of
 successive genotypes to GSS; \mjseqn{n} is the rank of the genotype when
 sorted according to the contribution to GSS and \mjseqn{A} is the asymptote
 of the curve (\mjseqn{A = 100}).

 The rate of increase in the successive contribution of genotypes to GSS can
 be computed by the following equation to find the point of inflection where
 the rate of increase starts declining.

 \mjseqn{\frac{\mathrm{d} y}{\mathrm{d} x} = by(A-y)}

 The number of accessions included till the peak or infection point are
 selected to constitute the core collection.

 }

}

Similarly for qualitative traits, standardized and independent variables or
factors can be obtained by Correspondence Analysis (CA) on complete
disjunctive table of genotype \mjseqn{\times} trait data or to be specific
Multiple Correspondence Analysis (MCA). In \code{rpcss}, this has also been
extended for data sets having both quantitative and qualitative traits by
implementing Factor Analysis for Mixed Data (FAMD) for obtaining standardized
and independent variables or factors.

In \code{rpcss}, PCA, MCA and FAMD are implemented via the
\code{\link[FactoMineR]{FactoMineR}} package.
\insertCite{le_FactoMineR_2008,husson_Exploratory_2017}{rpcss}.
}
\examples{

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if (requireNamespace('EvaluateCore', quietly = TRUE)) {

  suppressPackageStartupMessages(library(EvaluateCore))

  # Get data from EvaluateCore

  data("cassava_EC", package = "EvaluateCore")
  data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
  quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
             "AVPW", "ARSR", "SRDM")
  qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
            "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
            "PSTR")
  rownames(data) <- NULL

  # Convert qualitative data columns to factor
  data[, qual] <- lapply(data[, qual], as.factor)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Get core sets with PCSS (quantitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out1 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = NULL, eigen.threshold = NULL, size = 0.2,
                    var.threshold = 0.75)

  out1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
                    qualitative = qual, eigen.threshold = NULL,
                    size = 0.2, var.threshold = 0.75)

  out2

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (quantitative and qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out3 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = qual, eigen.threshold = NULL)

  out3

} else {
  message('Package "EvaluateCore" is required to run these examples.')
}

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[FactoMineR]{PCA}}, \code{\link[FactoMineR]{CA}} and
  \code{\link[FactoMineR]{FAMD}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset.pcss.core.R
\name{subset.pcss.core}
\alias{subset.pcss.core}
\title{Fetch the names of individuals/genotypes in the core set generated from
\code{pcss.core} Output}
\usage{
\method{subset}{pcss.core}(x, criterion = c("size", "variance", "logistic"), ...)
}
\arguments{
\item{x}{An object of class \code{pcss.core}.}

\item{criterion}{The core collection generation criterion. Either
\code{"size"}, \code{"variance"}, or \code{"logistic"}. See
\strong{Details}.}

\item{...}{Unused.}
}
\value{
The names of individuals/genotypes in the core collection as a
  character vector.
}
\description{
\code{subset.pcss.core} returns names of individuals/genotypes in the core
collection from \code{pcss.core} Output.
}
\details{
Use \code{"size"} to return names of individuals/genotypes in the core
collection according to the threshold \code{size} criterion or use
\code{"variance"} to return names according to the variability threshold
criterion or use  \code{"logistic"} to return names according to inflection
point of rate of progress of cumulative variability retained identified by
logistic regression.
}
\examples{

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if (requireNamespace('EvaluateCore', quietly = TRUE)) {

  suppressPackageStartupMessages(library(EvaluateCore))

  # Get data from EvaluateCore

  data("cassava_EC", package = "EvaluateCore")
  data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
  quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
             "AVPW", "ARSR", "SRDM")
  qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
            "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
            "PSTR")
  rownames(data) <- NULL

  # Convert qualitative data columns to factor
  data[, qual] <- lapply(data[, qual], as.factor)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# With quantitative data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out1 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = NULL, eigen.threshold = NULL, size = 0.2,
                    var.threshold = 0.75)

  # Core sets
  out1$cores.info

  # Fetch genotype names of core set by size criterion
  subset(x = out1, criterion = "size")

  # Fetch genotype names of core set by variance criterion
  subset(x = out1, criterion = "variance")

  # Fetch genotype names of core set by logistic regression criterion
  subset(x = out1, criterion = "logistic")

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
                    qualitative = qual, eigen.threshold = NULL,
                    size = 0.2, var.threshold = 0.75)

  # Core sets
  out2$cores.info

  # Fetch genotype names of core set by size criterion
  subset(x = out2, criterion = "size")

  # Fetch genotype names of core set by variance criterion
  subset(x = out2, criterion = "variance")

  # Fetch genotype names of core set by logistic regression criterion
  subset(x = out2, criterion = "logistic")
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (quantitative and qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out3 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = qual, eigen.threshold = NULL)

  # Core sets
  out3$cores.info

  # Fetch genotype names of core set by size criterion
  subset(x = out3, criterion = "size")

  # Fetch genotype names of core set by variance criterion
  subset(x = out3, criterion = "variance")

  # Fetch genotype names of core set by logistic regression criterion
  subset(x = out3, criterion = "logistic")

} else {
  message('Package "EvaluateCore" is required to run these examples.')
}

}
\seealso{
\code{\link[rpcss]{pcss.core}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screeplot.pcss.core.R
\name{screeplot.pcss.core}
\alias{screeplot.pcss.core}
\title{Plot Eigen values as a Scree Plot from \code{pcss.core} Output}
\usage{
\method{screeplot}{pcss.core}(x, ndim = NULL, show.values = TRUE, ...)
}
\arguments{
\item{x}{An object of class \code{pcss.core}.}

\item{ndim}{The number of eigen values to be plotted in the scree plot.}

\item{show.values}{If \code{TRUE}, the eigen values are shown in the plot as
annotation labels. Default is \code{TRUE}.}

\item{...}{Unused.}
}
\value{
The scree plot as a \code{ggplot} object.
}
\description{
\code{screeplot.pcss.core} generates a scree plot of eigen values from the
output of \code{pcss.core}.
}
\examples{

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if (requireNamespace('EvaluateCore', quietly = TRUE) &
    requireNamespace("factoextra", quietly = TRUE)) {

  # Get data from EvaluateCore

  data("cassava_EC", package = "EvaluateCore")
  data = cbind(Genotypes = rownames(cassava_EC), cassava_EC)
  quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
             "ARSR", "SRDM")
  qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
            "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
            "PSTR")
  rownames(data) <- NULL

  # Convert qualitative data columns to factor
  data[, qual] <- lapply(data[, qual], as.factor)


  library(FactoMineR)
  suppressPackageStartupMessages(library(factoextra))

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# With quantitative data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out1 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = NULL, eigen.threshold = NULL, size = 0.2,
                    var.threshold = 0.75)

  # Plot scree plot
  screeplot(x = out1)

  # Plot biplot with factoextra
  fviz_screeplot(out1$raw.out)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out2 <- pcss.core(data = data, names = "Genotypes", quantitative = NULL,
                    qualitative = qual, eigen.threshold = NULL,
                    size = 0.2, var.threshold = 0.75)

  # Plot scree plot
  screeplot(x = out2)

  # Plot biplot with factoextra
  fviz_screeplot(out2$raw.out)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Get core sets with PCSS (quantitative and qualitative data)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  out3 <- pcss.core(data = data, names = "Genotypes",
                    quantitative = quant,
                    qualitative = qual, eigen.threshold = NULL)

  # Plot scree plot
  screeplot(x = out3)

  # Plot biplot with factoextra
  fviz_screeplot(out3$raw.out)

} else {
  message('Packages "EvaluateCore" and "factoextra" are required to run these examples.')
}

}
\seealso{
\code{\link[rpcss]{pcss.core}},
  \code{\link[factoextra]{fviz_screeplot}}
}
